#!/usr/bin/python

"""
Configuration module for RO manager tests
"""

from rocommand import ro_settings

class ro_test_config:
    CONFIGDIR      = "config"
    ROBASEDIR      = "robase"
    NOBASEDIR      = "nobase"
    ROBOXURI       = "http://calatola.man.poznan.pl/robox/dropbox_accounts/1/ro_containers/2"
    ROBOXUSERNAME  = "Test User"
    ROBOXPASSWORD  = "d41d8cd98f00b204e9800998ecf8427e"
    ROBOXEMAIL     = "testuser@example.org"
    ROMANIFESTDIR  = ro_settings.MANIFEST_DIR
    ROMANIFESTFILE = ro_settings.MANIFEST_FILE
    ROMANIFESTPATH = ro_settings.MANIFEST_DIR+"/"+ro_settings.MANIFEST_FILE
    ROSRS_URI           = "http://sandbox.wf4ever-project.org/rosrs5"
    ROSRS_ACCESS_TOKEN  = "32801fc0-1df1-4e34-b"
# End.



# ro_metadata.py

"""
Research Object metadata (manifest and annotations) access class
"""

import sys
import os
import os.path
import re
import urllib
import urlparse
import logging

log = logging.getLogger(__name__)

import MiscLib.ScanDirectories

import rdflib
import rdflib.namespace
#from rdflib import Namespace, URIRef, BNode, Literal
# Set up to use SPARQL
#rdflib.plugin.register(
#    'sparql', rdflib.query.Processor,
#    'rdfextras.sparql.processor', 'Processor')
#rdflib.plugin.register(
#    'sparql', rdflib.query.Result,
#    'rdfextras.sparql.query', 'SPARQLQueryResult')

import ro_settings
from ro_namespaces import RDF, RO, ORE, AO, DCTERMS
from ro_uriutils import isFileUri, resolveUri, resolveFileAsUri, getFilenameFromUri, isLiveUri, retrieveUri
import ro_manifest
import ro_annotation


class ro_metadata(object):
    """
    Class for accessing RO metadata
    """

    def __init__(self, roconfig, roref, dummysetupfortest=False):
        """
        Initialize: read manifest from object at given directory into local RDF graph

        roconfig    is the research object manager configuration, supplied as a dictionary
        roref       a URI reference that refers to the Research Object to be accessed
        """
        self.roconfig = roconfig
        self.roref    = roref
        self.manifestgraph = None
        self.roannotations = None
        uri = resolveFileAsUri(roref)
        if not uri.endswith("/"): uri += "/"
        self.rouri    = rdflib.URIRef(uri)
        self.manifesturi  = self.getManifestUri()
        self.dummyfortest = dummysetupfortest
        self._loadManifest()
        # Get RO URI from manifest
        # May be different from computed value if manifest has absolute URI
        self.rouri = self.manifestgraph.value(None, RDF.type, RO.ResearchObject)
        return

    def _loadManifest(self):
        if self.manifestgraph: return self.manifestgraph
        self.manifestgraph = rdflib.Graph()
        if self.dummyfortest:
            # Fake minimal manifest graph for testing
            self.manifestgraph.add( (self.rouri, RDF.type, RO.ResearchObject) )
        else:
            # Read manifest graph
            self.manifestgraph.parse(self.manifesturi)
        return self.manifestgraph

    def _loadAnnotations(self):
        if self.roannotations: return self.roannotations
        # Assemble annotation graph
        # NOTE: the manifest itself is included as an annotation by the RO setup
        self._loadManifest()
        self.roannotations = rdflib.Graph()
        for (ann_node, subject) in self.manifestgraph.subject_objects(predicate=RO.annotatesAggregatedResource):
            ann_uri   = self.manifestgraph.value(subject=ann_node, predicate=AO.body)
            self._readAnnotationBody(ann_uri, self.roannotations)
        log.debug("roannotations graph:\n"+self.roannotations.serialize())
        return self.roannotations

    def updateManifest(self):
        """
        Write updated manifest file for research object
        """
        self.manifestgraph.serialize(
            destination=self.getManifestFilename(), format='xml',
            base=self.rouri, xml_base="..")
        return

    def addAggregatedResources(self, ro_file, recurse=True):
        """
        Scan a local directory and add files found to the RO aggregation
        """
        def notHidden(f):
            return re.match("\.|.*/\.", f) == None
        log.debug("addAggregatedResources: ref %s, file %s"%(self.roref, ro_file))
        self.getRoFilename()  # Check that we have one
        if ro_file.endswith(os.path.sep):
            ro_file = ro_file[0:-1]
        rofiles = [ro_file]
        if os.path.isdir(ro_file):
            rofiles = filter( notHidden,
                                MiscLib.ScanDirectories.CollectDirectoryContents(
                                    os.path.abspath(ro_file), baseDir=os.path.abspath(self.roref),
                                    listDirs=False, listFiles=True, recursive=recurse, appendSep=False)
                            )
        s = self.getRoUri()
        for f in rofiles:
            log.debug("- file %s"%f)
            stmt = (s, ORE.aggregates, self.getComponentUri(f))
            if stmt not in self.manifestgraph: self.manifestgraph.add(stmt)
        self.updateManifest()
        return

    def getAggregatedResources(self):
        """
        Returns iterator over all resources aggregated by a manifest.

        Each value returned by the iterator is an aggregated resource URI
        """
        log.debug("getAggregatedResources: uri %s"%(self.rouri))
        for r in self.manifestgraph.objects(subject=self.rouri, predicate=ORE.aggregates):
            yield r
        return

    def _createAnnotationBody(self, roresource, attrdict, defaultType="string"):
        """
        Create a new annotation body for a single resource in a research object, based
        on a supplied graph value.

        Existing annotations for the same resource are not touched; if an annotation is being
        added or replaced, it is the calling program'sresponsibility to update the manifest to
        reference the active annotations.  A new name is allocated for the created annotation,
        graph which is returned as the result of this function.

        roresource  is the name of the Research Object component to be annotated,
                    possibly relative to the RO root directory.
        attrdict    is a dictionary of attributes to be saved in the annotation body.
                    Dictionary keys are attribute names that can be resolved via
                    ro_annotation.getAnnotationByName.

        Returns the name of the annotation body created relative to the RO directory.
        """
        af = ro_annotation.createAnnotationBody(
            self.roconfig, self.getRoFilename(), roresource, attrdict, defaultType)
        return os.path.join(ro_settings.MANIFEST_DIR+"/", af)

    def _createAnnotationGraphBody(self, roresource, anngraph):
        """
        Create a new annotation body for a single resource in a research object, based
        on a supplied graph value.

        Existing annotations for the same resource are not touched; if an annotation is being
        added or replaced, it is the calling program'sresponsibility to update the manifest to
        reference the active annotations.  A new name is allocated for the created annotation,
        graph which is returned as the result of this function.

        roresource  is the name of the Research Object component to be annotated,
                    possibly relative to the RO root directory.
        anngraph    is an annotation graph that is to be saved.

        Returns the name of the annotation body created relative to the RO
        manifest and metadata directory.
        """
        af = ro_annotation.createAnnotationGraphBody(
            self.roconfig, self.getRoFilename(), roresource, anngraph)
        return os.path.join(ro_settings.MANIFEST_DIR+"/", af)

    def _readAnnotationBody(self, annotationref, anngr=None):
        """
        Read annotation body from indicated resource, return RDF Graph of annotation values.

        annotationref   is a URI reference of an annotation, possibly relative to the RO base URI
                        (e.g. as returned by _createAnnotationBody method).
        """
        log.debug("_readAnnotationBody %s"%(annotationref))
        annotationuri    = self.getComponentUri(annotationref)
        annotationformat = "xml"
        # Look at file extension to figure format
        # (rdflib.Graph.parse says;
        #   "used if format can not be determined from the source")
        if re.search("\.(ttl|n3)$", annotationuri): annotationformat="n3"
        if anngr == None:
            log.debug("_readAnnotationBody: new graph")
            anngr = rdflib.Graph()
        try:
            anngr.parse(annotationuri, format=annotationformat)
            log.debug("_readAnnotationBody parse %s, len %i"%(annotationuri, len(anngr)))
        except IOError, e:
            log.debug("_readAnnotationBody "+annotationref+", "+repr(e))
            anngr = None
        return anngr

    def _addAnnotationBodyToRoGraph(self, rofile, annfile):
        """
        Add a new annotation body to an RO graph

        rofile      is the research object file being annotated
        annfile     is the file name of the annotation body to be added,
                    possibly relative to the RO URI
        """
        # <ore:aggregates>
        #   <ro:AggregatedAnnotation>
        #     <ro:annotatesAggregatedResource rdf:resource="data/UserRequirements-astro.ods" />
        #     <ao:body rdf:resource=".ro/(annotation).rdf" />
        #   </ro:AggregatedAnnotation>
        # </ore:aggregates>
        log.debug("_addAnnotationBodyToRoGraph annfile %s"%(annfile))
        ann = rdflib.BNode()
        self.manifestgraph.add((ann, RDF.type, RO.AggregatedAnnotation))
        self.manifestgraph.add((ann, RO.annotatesAggregatedResource,
            self.getComponentUri(rofile)))
        self.manifestgraph.add((ann, AO.body,
            self.getComponentUri(annfile)))
        self.manifestgraph.add((self.getRoUri(), ORE.aggregates, ann))
        return

    def _removeAnnotationBodyFromRoGraph(self, annbody):
        """
        Remove references to an annotation body from an RO graph

        annbody     is the the annotation body node to be removed
        """
        self.manifestgraph.remove((annbody, None,           None   ))
        self.manifestgraph.remove((None,    ORE.aggregates, annbody))
        return

    def addSimpleAnnotation(self, rofile, attrname, attrvalue, defaultType="string"):
        """
        Add a simple annotation to a file in a research object.

        rofile      names the file or resource to be annotated, possibly relative to the RO.
        attrname    names the attribute in a form recognized by getAnnotationByName
        attrvalue   is a value to be associated with the attribute
        """
        ro_dir   = self.getRoFilename()
        annfile  = self._createAnnotationBody(rofile, {attrname: attrvalue}, defaultType)
        self._addAnnotationBodyToRoGraph(rofile, annfile)
        self.updateManifest()
        return

    def removeSimpleAnnotation(self, rofile, attrname, attrvalue):
        """
        Remove a simple annotation or multiple matching annotations a research object.

        rofile      names the annotated file or resource, possibly relative to the RO.
        attrname    names the attribute in a form recognized by getAnnotationByName
        attrvalue   is the attribute value to be deleted, or Nomne to delete all vaues
        """
        ro_dir    = self.getRoFilename()
        ro_graph  = self._loadManifest()
        subject     = self.getComponentUri(rofile)
        (predicate,valtype) = ro_annotation.getAnnotationByName(self.roconfig, attrname)
        val         = attrvalue and ro_annotation.makeAnnotationValue(attrvalue, valtype)
        add_annotations    = []
        remove_annotations = []
        log.debug("removeSimpleAnnotation subject %s, predicate %s, val %s"%
                  (str(subject), str(predicate), val))
        # Scan for annotation graph resourcxes containing this annotation
        for ann_node in ro_graph.subjects(predicate=RO.annotatesAggregatedResource, object=subject):
            ann_uri   = ro_graph.value(subject=ann_node, predicate=AO.body)
            log.debug("removeSimpleAnnotation ann_uri %s"%(str(ann_uri)))
            if self.isRoMetadataRef(ann_uri):
                ann_graph = self._readAnnotationBody(self.getComponentUriRel(ann_uri))
                log.debug("removeSimpleAnnotation ann_graph %s"%(ann_graph))
                if (subject, predicate, val) in ann_graph:
                    ann_graph.remove((subject, predicate, val))
                    if (subject, None, None) in ann_graph:
                        # Triples remain in annotation body: write new body and update RO graph
                        ann_name = self._createAnnotationGraphBody(rofile, ann_graph)
                        remove_annotations.append(ann_node)
                        add_annotations.append(ann_name)
                    else:
                        # Remove annotation from RO graph
                        remove_annotations.append(ann_node)
        # Update RO manifest graph if needed
        if add_annotations or remove_annotations:
            for a in remove_annotations:
                self._removeAnnotationBodyFromRoGraph(a)
            for a in add_annotations:
                self._addAnnotationBodyToRoGraph(rofile, a)
            self.updateManifest()
        return

    def replaceSimpleAnnotation(self, rofile, attrname, attrvalue):
        """
        Replace a simple annotation in a research object.

        rofile      names the file or resource to be annotated, possibly relative to the RO.
        attrname    names the attribute in a form recognized by getAnnotationByName
        attrvalue   is a new value to be associated with the attribute
        """
        ro_dir   = self.getRoFilename()
        ro_graph = self._loadManifest()
        subject  = self.getComponentUri(rofile)
        (predicate,valtype) = ro_annotation.getAnnotationByName(self.roconfig, attrname)
        log.debug("Replace annotation: subject %s, predicate %s, value %s"%
                  (repr(subject), repr(predicate), repr(attrvalue)))
        ro_graph.remove((subject, predicate, None))
        ro_graph.add((subject, predicate,
                      ro_annotation.makeAnnotationValue(attrvalue, valtype)))
        self.updateManifest()
        return

    def addGraphAnnotation(self, rofile, graph):
        """
        Add an annotation graph for a named resource.  Unlike addSimpleAnnotation, this
        method adds an annotation to the manifest using an existing RDF graph (which is
        presumably itself in the RO structure).

        rofile      names the file or resource to be annotated, possibly relative to the RO.
                    Note that no checks are performed to ensure that the graph itself actually
                    refers to this resource - that's up the the creator of the graph.  This
                    identifies the resourfce with which the annotation body is associated in
                    the RO manifest.
        graph       names the file or resource containing the annotation body.
        """
        ro_graph = self._loadManifest()
        ann = rdflib.BNode()
        ro_graph.add((ann, RDF.type, RO.AggregatedAnnotation))
        ro_graph.add((ann, RO.annotatesAggregatedResource, self.getComponentUri(rofile)))
        ro_graph.add((ann, AO.body, self.getComponentUri(graph)))
        ro_graph.add((self.getRoUri(), ORE.aggregates, ann))
        self.updateManifest()
        return

    def iterateAnnotations(self, subject=None, property=None):
        """
        Returns an iterator over annotations of the current RO that match th
        supplied subject and/or property.
        """
        log.debug("iterateAnnotations s:%s, p:%s"%(str(subject),str(property)))
        ann_graph = self._loadAnnotations()
        for (s, p, v) in ann_graph.triples((subject, property, None)):
            if not isinstance(s, rdflib.BNode):
                log.debug("Triple: %s %s %s"%(s,p,v))
                yield (s, p, v)
        return

    def getRoAnnotations(self):
        """
        Returns iterator over annotations applied to the RO as an entity.

        Each value returned by the iterator is a (subject,predicate,object) triple.
        """
        return self.iterateAnnotations(subject=self.getRoUri())

    def getFileAnnotations(self, rofile):
        """
        Returns iterator over annotations applied to a specified component in the RO

        Each value returned by the iterator is a (subject,predicate,object) triple.
        """
        return self.iterateAnnotations(subject=self.getComponentUri(rofile))

    def getAllAnnotations(self):
        """
        Returns iterator over all annotations associated with the RO

        Each value returned by the iterator is a (subject,predicate,object) triple.
        """
        return self.iterateAnnotations()

    def getAnnotationValues(self, rofile, attrname):
        """
        Returns iterator over annotation values for given subject and attribute
        """
        (predicate,valtype) = ro_annotation.getAnnotationByName(self.roconfig, attrname)
        return ( v for (s,p,v) in self.iterateAnnotations(subject=self.getComponentUri(rofile), property=predicate) )

    def queryAnnotations(self, query, initBindings={}):
        """
        Runs a query over the combined annotation graphs (including the manifest)
        and returns True or False (for ASK queries) or a list of doctionaries of
        query results (for SELECT queries).
        """
        ann_gr = self._loadAnnotations()
        resp = ann_gr.query(query,initBindings=initBindings)
        if resp.type == 'ASK':
            return resp.askAnswer
        elif resp.type == 'SELECT':
            return resp.bindings
        else:
            assert False, "Unexpected query response type %s"%resp.type
        return None

    def showAnnotations(self, annotations, outstr):
        ro_annotation.showAnnotations(self.roconfig, self.getRoFilename(), annotations, outstr)
        return

    # Support methods for accessing the manifest graph

    def _getRoManifestGraph(self):
        """
        Returns RDF graph containing RO manifest
        """
        return self.manifestgraph

    def roManifestContains(self, stmt):
        """
        Returns True if the RO manifest contains a statement matching the supplied triple.
        """
        return stmt in self.manifestgraph

    def getResourceValue(self, resource, predicate):
        """
        Returns value for resource whose URI is supplied assocfiated with indicated predicate
        """
        return self.manifestgraph.value(subject=resource, predicate=predicate, object=None)

    def getResourceType(self, resource):
        """
        Returns type of resource whose URI is supplied
        """
        return self.getResourceValue(resource, RDF.type)

    def getRoMetadataDict(self):
        """
        Returns dictionary of metadata about the RO from the manifest graph
        """
        strsubject = ""
        if isinstance(self.rouri, rdflib.URIRef): strsubject = str(self.rouri)
        manifestDict = {
            'ropath':         self.getRoFilename(),
            'rouri':          strsubject,
            'roident':        self.getResourceValue(self.rouri, DCTERMS.identifier  ),
            'rotitle':        self.getResourceValue(self.rouri, DCTERMS.title       ),
            'rocreator':      self.getResourceValue(self.rouri, DCTERMS.creator     ),
            'rocreated':      self.getResourceValue(self.rouri, DCTERMS.created     ),
            'rodescription':  self.getResourceValue(self.rouri, DCTERMS.description ),
            }
        return manifestDict

    # Support methods for accessing RO and component URIs

    def getRoRef(self):
        """
        Returns RO URI reference supplied (which may be a local file directory string)
        """
        return self.roref

    def getRoUri(self):
        return self.rouri

    def getComponentUri(self, path):
        return rdflib.URIRef(urlparse.urljoin(str(self.getRoUri()), path))

    def getComponentUriRel(self, path):
        file_uri = urlparse.urlunsplit(urlparse.urlsplit(str(self.getComponentUri(path))))
        ro_uri   = urlparse.urlunsplit(urlparse.urlsplit(str(self.getRoUri())))
        if ro_uri is not None and file_uri.startswith(ro_uri):
            file_uri_rel = file_uri.replace(ro_uri, "", 1)
        else:
            file_uri_rel = path
        return rdflib.URIRef(file_uri_rel)

    def isRoMetadataRef(self, uri):
        """
        Test if supplied URI is a reference to the current RO metadata area
        """
        urirel = self.getComponentUriRel(uri)
        return str(urirel).startswith(ro_settings.MANIFEST_DIR+"/")

    def getManifestUri(self):
        return self.getComponentUri(ro_settings.MANIFEST_DIR+"/"+ro_settings.MANIFEST_FILE)

    def isLocalFileRo(self):
        """
        Test current RO URI to see if it is a local file system reference
        """
        return isFileUri(self.getRoUri())

    def getRoFilename(self):
        return getFilenameFromUri(self.getRoUri())

    def getManifestFilename(self):
        """
        Return manifesrt file name: used for local updates
        """
        return os.path.join(self.getRoFilename(), ro_settings.MANIFEST_DIR+"/",
                            ro_settings.MANIFEST_FILE)

# End.




#!/bin/env python

import sys
import re

p = "{ "
for l in sys.stdin.readlines():
    m = re.match(r"PREFIX\s*(\w*):\s*<([^>]*)>",l)
    if m:
        print '''%s"%s":%s"%s"'''%(p, m.group(1), (" "*(10-len(m.group(1)))), m.group(2))
        p = ", "
print "}"




#!/usr/bin/python

"""
Module to test RO manager annotation commands

See: http://www.wf4ever-project.org/wiki/display/docs/RO+management+tool
"""

import os, os.path
import sys
import re
import shutil
import unittest
import logging
import datetime
import StringIO
try:
    # Running Python 2.5 with simplejson?
    import simplejson as json
except ImportError:
    import json

log = logging.getLogger(__name__)

if __name__ == "__main__":
    # Add main project directory and ro manager directories at start of python path
    sys.path.insert(0, "../..")
    sys.path.insert(0, "..")

import rdflib

from MiscLib import TestUtils

from rocommand import ro
from rocommand import ro_utils
from rocommand import ro_manifest
from rocommand import ro_annotation
from rocommand.ro_namespaces import RDF, RDFS, DCTERMS, RO, AO, ORE

from TestConfig import ro_test_config
from StdoutContext import SwitchStdout

import TestROSupport

# Base directory for RO tests in this module
testbase = os.path.dirname(os.path.abspath(__file__))

# Local ro_config for testing
ro_config = {
    "annotationTypes": ro_annotation.annotationTypes
    }

class TestAnnotations(TestROSupport.TestROSupport):
    """
    Test ro annotation commands
    """
    def setUp(self):
        super(TestAnnotations, self).setUp()
        return

    def tearDown(self):
        super(TestAnnotations, self).tearDown()
        return

    # Actual tests follow

    def testNull(self):
        assert True, 'Null test failed'

    def testAnnotate(self):
        """
        Annotate file in created RO

        ro annotate file attribute-name [ attribute-value ]
        """
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        args = [
            "ro", "annotate", rodir+"/"+"subdir1/subdir1-file.txt", "title", "subdir1-file title",
            "-v",
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        outtxt = self.outstr.getvalue()
        assert status == 0, outtxt
        log.debug("outtxt %s"%outtxt)
        #self.assertRegexpMatches(outtxt, "annotation.*dc:title")
        # Read manifest and check for annotation
        values = ro_annotation._getAnnotationValues(ro_config, rodir, "subdir1/subdir1-file.txt", "title")
        self.assertEquals(values.next(), rdflib.Literal("subdir1-file title"))
        self.assertRaises(StopIteration, values.next)
        # Clean up
        self.deleteTestRo(rodir)
        return

    def annotateTest(self, anntype, annvalue, anntypeuri, annexpect):
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        args = [
            "ro", "annotate", rodir+"/"+"subdir1/subdir1-file.txt", anntype, annvalue,
            "-v",
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        outtxt = self.outstr.getvalue()
        assert status == 0, outtxt
        self.assertEqual(outtxt.count("ro annotate"), 1)
        # Read manifest and check for annotation
        annotations = ro_annotation._getFileAnnotations(rodir, "subdir1/subdir1-file.txt")
        resourceuri = ro_manifest.getComponentUri(rodir, "subdir1/subdir1-file.txt")
        expected_annotations = (
            [ (resourceuri, anntypeuri, rdflib.Literal(annexpect))
            ])
        for i in range(1):
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        self.deleteTestRo(rodir)
        return

    # Other annotation types to add (cf. http://wf4ever.github.com/labs/ro-annotator/mockups/1/index.html)

    def testAnnotateType(self):
        self.annotateTest("type", "file type", DCTERMS.type, "file type")
        return

    def testAnnotateKeywords(self):
        self.annotateTest("keywords", "foo,bar", DCTERMS.subject, "foo,bar")  #@@TODO: make multiples
        return

    def testAnnotateDescription(self):
        descr = """
            Multiline
            description
            """
        self.annotateTest("description", descr, DCTERMS.description, descr)
        return

    def testAnnotateCreated(self):
        #@@TODO: use for creation date/time of file
        created = "2011-09-14T12:00:00"
        self.annotateTest("created", created, DCTERMS.created, created)
        return

    def testAnnotateTypeUri(self):
        anntypestr = "http://example.org/annotationtype"
        anntypeuri = rdflib.URIRef(anntypestr)
        annvalue   = "Annotation value"
        self.annotateTest(anntypestr, annvalue, anntypeuri, annvalue)
        return

    def testAnnotateTypeCurie(self):
        anntypestr = "rdfs:comment"
        anntypeuri = RDFS.comment
        annvalue   = "Annotation value"
        self.annotateTest(anntypestr, annvalue, anntypeuri, annvalue)
        return

    # @@TODO: Test use of CURIE as type

    def annotateMultiple(self, rodir, rofile, annotations):
        with SwitchStdout(self.outstr):
            for a in annotations:
                args = ["ro", "annotate", rofile, a["atypename"], a["avalue"]]
                status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
                outtxt = self.outstr.getvalue()
                assert status == 0, outtxt
        # Reset output stream buffer closed
        self.outstr = StringIO.StringIO()
        return

    def testAnnotateMultiple(self):
        rodir  = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        rofile = rodir+"/"+"subdir1/subdir1-file.txt"
        define_annotations = (
            [ {"atypename": "type",        "avalue":"atype",    "atypeuri":DCTERMS.type,        "aexpect":"atype" }
            , {"atypename": "keywords",    "avalue":"asubj",    "atypeuri":DCTERMS.subject,     "aexpect":"asubj" }
            , {"atypename": "description", "avalue":"adesc",    "atypeuri":DCTERMS.description, "aexpect":"adesc" }
            , {"atypename": "format",      "avalue":"aformat",  "atypeuri":DCTERMS.format,      "aexpect":"aformat" }
            , {"atypename": "title",       "avalue":"atitle",   "atypeuri":DCTERMS.title,       "aexpect":"atitle" }
            , {"atypename": "created",     "avalue":"acreated", "atypeuri":DCTERMS.created,     "aexpect":"acreated" }
            #, {"atypename": ..., "avalue":..., "atypeuri":..., "aexpect":... }
            #, {"atypename": ..., "avalue":..., "atypeuri":..., "aexpect":... }
            ])
        self.annotateMultiple(rodir, rofile, define_annotations)
        # Read manifest and check for annotation
        annotations = ro_annotation._getFileAnnotations(rodir, "subdir1/subdir1-file.txt")
        resourceuri = ro_manifest.getComponentUri(rodir, "subdir1/subdir1-file.txt")
        expected_annotations = (
            [ (resourceuri, a["atypeuri"], rdflib.Literal(a["aexpect"]))
                for a in define_annotations
            ])
        for i in range(6):
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        # Clean up
        self.deleteTestRo(rodir)
        return

    # Test annotation display
    def testAnnotationDisplayRo(self):
        # Construct annotated RO
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        rofile = rodir+"/"+"subdir1/subdir1-file.txt"
        # Display annotations
        args = [ "ro", "annotations", rodir+"/"
               , "-v"
               ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        outtxt = self.outstr.getvalue()
        assert status == 0, "Status %d, outtxt: %s"%(status,outtxt)
        log.debug("status %d, outtxt: %s"%(status, outtxt))
        self.assertEqual(outtxt.count("ro annotations"), 1)
        #self.assertRegexpMatches(outtxt, "((name))")
        self.assertRegexpMatches(outtxt, "title.*RO test annotation")
        self.assertRegexpMatches(outtxt, "created.*\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\d")
        self.assertRegexpMatches(outtxt, "description.*RO test annotation")
        self.assertRegexpMatches(outtxt, "rdf:type.*<%s>"%(RO.ResearchObject))
        self.assertRegexpMatches(outtxt, "dcterms:identifier.*ro-testRoAnnotate")
        self.assertRegexpMatches(outtxt, "dcterms:creator.*Test User")
        self.deleteTestRo(rodir)
        return

    def testAnnotationDisplayFile(self):
        # Construct annotated RO
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        rofile = rodir+"/"+"subdir1/subdir1-file.txt"
        annotations = (
            [ {"atypename": "type",        "avalue":"atype",    "atypeuri":DCTERMS.type,        "aexpect":"atype" }
            , {"atypename": "title",       "avalue":"atitle",   "atypeuri":DCTERMS.title,       "aexpect":"atitle" }
            ])
        self.annotateMultiple(rodir, rofile, annotations)
        # Display annotations
        args = [ "ro", "annotations", rofile
               , "-v"
               ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        outtxt = self.outstr.getvalue()
        assert status == 0, outtxt
        log.debug("outtxt: %s"%(outtxt))
        self.assertEqual(outtxt.count("ro annotations"), 1)
        self.assertRegexpMatches(outtxt, "\n<subdir1/subdir1-file.txt>")
        self.assertRegexpMatches(outtxt, "type.*atype")
        self.assertRegexpMatches(outtxt, "title.*atitle")
        self.deleteTestRo(rodir)
        return

    # Test annotate with graph
    def testAnnotateWithGraph(self):
        rodir  = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        rofile = rodir+"/"+"subdir1/subdir1-file.txt"
        define_annotations = (
            [ {"atypename": "type",        "avalue":"atype",    "atypeuri":DCTERMS.type,        "aexpect":"atype" }
            , {"atypename": "keywords",    "avalue":"asubj",    "atypeuri":DCTERMS.subject,     "aexpect":"asubj" }
            , {"atypename": "description", "avalue":"adesc",    "atypeuri":DCTERMS.description, "aexpect":"adesc" }
            , {"atypename": "format",      "avalue":"aformat",  "atypeuri":DCTERMS.format,      "aexpect":"aformat" }
            , {"atypename": "title",       "avalue":"atitle",   "atypeuri":DCTERMS.title,       "aexpect":"atitle" }
            , {"atypename": "created",     "avalue":"acreated", "atypeuri":DCTERMS.created,     "aexpect":"acreated" }
            ])
        # Create annotation graph file and apply annotations
        annotation_graph = rdflib.Graph()
        resourceuri = ro_manifest.getComponentUri(rodir, "subdir1/subdir1-file.txt")
        for ann in define_annotations:
            annotation_graph.add( (resourceuri, ann["atypeuri"], rdflib.Literal(ann["aexpect"])) )
        annotation_graph_filename = os.path.join(os.path.abspath(rodir), "annotate-subdir1-file.txt.rdf")
        annotation_graph.serialize(annotation_graph_filename,
            format='xml', base=ro_manifest.getRoUri(rodir), xml_base="")
        args = ["ro", "annotate", rofile, "-g", annotation_graph_filename ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        outtxt = self.outstr.getvalue()
        assert status == 0, outtxt
        # Read manifest and check for annotation
        annotations = ro_annotation._getFileAnnotations(rodir, "subdir1/subdir1-file.txt")
        expected_annotations = (
            [ (resourceuri, a["atypeuri"], rdflib.Literal(a["aexpect"]))
                for a in define_annotations
            ])
        for i in range(6):
            next = annotations.next()
            if ( next not in expected_annotations):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        # Clean up
        self.deleteTestRo(rodir)
        return

    # Test annotate with non-existent graph (make nsure it doesn't all fall over)
    def testAnnotateWithNotExistentGraph(self):
        rodir  = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        rofile = rodir+"/"+"subdir1/subdir1-file.txt"
        # Apply non-exietent graph annotation
        annotation_graph_filename = os.path.join(os.path.abspath(rodir), "annotate-none.rdf")
        rouri = ro_manifest.getRoUri(rodir)
        args = ["ro", "annotate", rodir+"/", "-g", annotation_graph_filename ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        outtxt = self.outstr.getvalue()
        assert status == 0, outtxt
        # Read manifest and check for annotation
        annotations = ro_annotation.getAllAnnotations(rodir)
        expected_annotations = (
            [ ( rouri, DCTERMS.identifier,  rdflib.Literal('ro-testRoAnnotate')  )
            , ( rouri, DCTERMS.description, rdflib.Literal('RO test annotation') )
            , ( rouri, DCTERMS.title,       rdflib.Literal('RO test annotation') )
            , ( rouri, DCTERMS.creator,     rdflib.Literal('Test User') )
            , ( rouri, RDF.type,            RO.ResearchObject )
            ])
        for i in range(8):
            next = annotations.next()
            if (next not in expected_annotations
                and not isinstance(next[2], rdflib.BNode)
                and not next[1] == DCTERMS.created):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        # Clean up
        self.deleteTestRo(rodir)
        return

    # Test display of annotations for entire RO

    # @@TODO: Test annotations shown in RO listing

    # Sentinel/placeholder tests

    def testUnits(self):
        assert (True)

    def testComponents(self):
        assert (True)

    def testIntegration(self):
        assert (True)

    def testPending(self):
        assert (False), "Pending tests follow"

# Assemble test suite

def getTestSuite(select="unit"):
    """
    Get test suite

    select  is one of the following:
            "unit"      return suite of unit tests only
            "component" return suite of unit and component tests
            "all"       return suite of unit, component and integration tests
            "pending"   return suite of pending tests
            name        a single named test to be run
    """
    testdict = {
        "unit":
            [ "testUnits"
            , "testNull"
            , "testAnnotate"
            , "testAnnotateType"
            , "testAnnotateKeywords"
            , "testAnnotateDescription"
            , "testAnnotateCreated"
            , "testAnnotateTypeUri"
            , "testAnnotateTypeCurie"
            , "testAnnotateMultiple"
            , "testAnnotationDisplayRo"
            , "testAnnotationDisplayFile"
            , "testAnnotateWithGraph"
            , "testAnnotateWithNotExistentGraph"
            ],
        "component":
            [ "testComponents"
            ],
        "integration":
            [ "testIntegration"
            ],
        "pending":
            [ "testPending"
            ]
        }
    return TestUtils.getTestSuite(TestAnnotations, testdict, select=select)

if __name__ == "__main__":
    TestUtils.runTests("TestAnnotations.log", getTestSuite, sys.argv)

# End.



# ro_minim.py

"""
Research Object minimum information model access functions
"""

import re
import urlparse
import logging

log = logging.getLogger(__name__)

import rdflib
import rdflib.namespace

from rocommand import ro_manifest
from rocommand import ro_namespaces
from rocommand.ro_namespaces import RDF, RDFS

minimnsuri = rdflib.URIRef("http://purl.org/minim/minim#")
MINIM      = ro_namespaces.makeNamespace(minimnsuri,
            [ "Constraint", "Model", "Requirement", "RequirementRule"
            , "SoftwareEnvironmentRule", "DataRequirementRule", "ContentMatchRequirementRule"
            , "hasConstraint", "forPurpose", "onResource", "toModel"
            , "hasMustRequirement", "hasShouldRequirement", "hasMayRequirement", "hasRequirement"
            , "derives", "reports", "isDerivedBy"
            , "show", "showpass", "showfail"
            , "aggregates"
            , "command", "response"
            , "forall", "exists", "aggregatesTemplate", "isLiveTemplate"
            , "minimallySatisfies", "nominallySatisfies", "fullySatisfies"
            # Result properties
            , "satisfied", "missingMay", "missingShould", "missingMust"
            , "testedConstraint", "testedPurpose", "testedTarget"
            , "minimUri", "modelUri"
            #, "tryRule"
            , "tryRequirement"
            ])

resultnsuri = rdflib.URIRef("http://www.w3.org/2001/sw/DataAccess/tests/result-set#")
RESULT      = ro_namespaces.makeNamespace(resultnsuri,
            [ "binding", "variable", "value"
            ])

def getElementUri(minimbase, elemname):
    """
    Returns URI of element in Minim file
    
    minimbase   is the URI of the Minim file containing the required element
    elemname    is the relative URI of the element - commonly a fragment identifier
    """
    return rdflib.URIRef(urlparse.urljoin(str(minimbase), elemname))

def readMinimGraph(minimuri):
    """
    Read Minim file, return RDF Graph.
    """
    minimgraph = rdflib.Graph()
    minimgraph.parse(minimuri)
    return minimgraph

def getConstraints(minimgraph):
    for (target, constraint) in minimgraph.subject_objects(predicate=MINIM.hasConstraint):
        c = {'target': target, 'uri': constraint}
        c['purpose']  = minimgraph.value(subject=constraint, predicate=MINIM.forPurpose)
        c['resource'] = minimgraph.value(subject=constraint, predicate=MINIM.onResource)
        c['model']    = minimgraph.value(subject=constraint, predicate=MINIM.toModel)
        yield c
    return

def getConstraint(minimgraph, roref, target_ref, purpose_regex_string):
    target  = target_ref and ro_manifest.getComponentUri(roref, target_ref)
    purpose = purpose_regex_string and re.compile(purpose_regex_string)
    for c in getConstraints(minimgraph):
        if ( ( not target  or target == c['target'] ) and
             ( not purpose or purpose.match(c['purpose']) ) ):
            return c
    return None

def getModels(minimgraph, modeluri=None):
    for (model, p, o) in minimgraph.triples( (modeluri, RDF.type, MINIM.Model) ):
        m = {'uri': model}
        m['label']   = minimgraph.value(subject=model, predicate=RDFS.label)
        m['comment'] = minimgraph.value(subject=model, predicate=RDFS.comment)
        yield m
    return

def getModel(minimgraph, modeluri):
    for m in getModels(minimgraph, modeluri=modeluri):
        return m
    return None

def getRequirements(minimgraph, modeluri):
    def matchRequirement((s, p, o), reqp, reqval):
        req = None
        if p == reqp:
            req = (
                { 'uri':    o
                , 'model':  s
                , 'level':  reqval
                , 'label':  minimgraph.value(subject=o, predicate=RDFS.label)
                })
            ruleuri = minimgraph.value(subject=o, predicate=MINIM.isDerivedBy)
            assert ruleuri, "Requirement %s has no minim:isDerivedBy rule"%(str(o))
            rule = (
                { 'derives':    minimgraph.value(subject=ruleuri, predicate=MINIM.derives) 
                , 'show':       minimgraph.value(subject=ruleuri, predicate=MINIM.show) 
                , 'showpass':   minimgraph.value(subject=ruleuri, predicate=MINIM.showpass)
                , 'showfail':   minimgraph.value(subject=ruleuri, predicate=MINIM.showfail)
                })
            ruletype = minimgraph.value(subject=ruleuri, predicate=RDF.type)
            if ruletype == MINIM.DataRequirementRule:
                rule['aggregates']  = minimgraph.value(subject=ruleuri, predicate=MINIM.aggregates)
                req['datarule'] = rule
            elif ruletype == MINIM.SoftwareEnvironmentRule:
                rule['command']  = minimgraph.value(subject=ruleuri, predicate=MINIM.command)
                rule['response'] = minimgraph.value(subject=ruleuri, predicate=MINIM.response)
                req['softwarerule'] = rule
            elif ruletype == MINIM.ContentMatchRequirementRule:
                rule['forall']   = minimgraph.value(subject=ruleuri, predicate=MINIM.forall)
                rule['exists']   = minimgraph.value(subject=ruleuri, predicate=MINIM.exists)
                rule['template'] = minimgraph.value(subject=ruleuri, predicate=MINIM.aggregatesTemplate)
                rule['islive']   = minimgraph.value(subject=ruleuri, predicate=MINIM.isLiveTemplate)
                req['contentmatchrule'] = rule
            else:
                assert False, "Unrecognized rule type %s for requirement %s"%(str(ruletype), str(o))
        return req
    for stmt in minimgraph.triples( (modeluri, None, None) ):
        pred_level_list = (
            [ (MINIM.hasMustRequirement, "MUST")
            , (MINIM.hasShouldRequirement, "SHOULD")
            , (MINIM.hasMayRequirement, "MAY")
            ])
        for (pred, level) in pred_level_list:
            r = matchRequirement(stmt, pred, level)
            if r:
                yield r
                break
    return

# End.



# Early, and incomplete implementation of -04.
#
import re
import urllib

RESERVED = ":/?#[]@!$&'()*+,;="
OPERATOR = "+./;?|!@"
EXPLODE = "*+"
MODIFIER = ":^"
TEMPLATE = re.compile("{([^\}]+)}")

def _tostring(varname, value, explode, operator, safe=""):
  if type(value) == type([]):
    if explode == "+":
      return ",".join([varname + "." + urllib.quote(x, safe) for x in value])
    else:
      return ",".join([urllib.quote(x, safe) for x in value])
  if type(value) == type({}):
    keys = value.keys()
    keys.sort()
    if explode == "+":
      return ",".join([varname + "." + urllib.quote(key, safe) + "," + urllib.quote(value[key], safe) for key in keys])
    else:
      return ",".join([urllib.quote(key, safe) + "," + urllib.quote(value[key], safe) for key in keys])
  else:
    return urllib.quote(value, safe)


def _tostring_path(varname, value, explode, operator, safe=""):
  joiner = operator
  if type(value) == type([]):
    if explode == "+":
      return joiner.join([varname + "." + urllib.quote(x, safe) for x in value])
    elif explode == "*":
      return joiner.join([urllib.quote(x, safe) for x in value])
    else:
      return ",".join([urllib.quote(x, safe) for x in value])
  elif type(value) == type({}):
    keys = value.keys()
    keys.sort()
    if explode == "+":
      return joiner.join([varname + "." + urllib.quote(key, safe) + joiner + urllib.quote(value[key], safe) for key in keys])
    elif explode == "*":
      return joiner.join([urllib.quote(key, safe) + joiner + urllib.quote(value[key], safe) for key in keys])
    else:
      return ",".join([urllib.quote(key, safe) + "," + urllib.quote(value[key], safe) for key in keys])
  else:
    if value:
      return urllib.quote(value, safe)
    else:
      return ""


def _tostring_semi(varname, value, explode, operator, safe=""):
  joiner = operator
  if operator == "?":
    joiner = "&"
  if type(value) == type([]):
    if explode == "+":
      return joiner.join([varname + "=" + urllib.quote(x, safe) for x in value])
    elif explode == "*":
      return joiner.join([urllib.quote(x, safe) for x in value])
    else:
      return ",".join([urllib.quote(x, safe) for x in value])
  elif type(value) == type({}):
    keys = value.keys()
    keys.sort()
    if explode == "+":
      return joiner.join([varname + "." + urllib.quote(key, safe) + "=" + urllib.quote(value[key], safe) for key in keys])
    elif explode == "*":
      return joiner.join([urllib.quote(key, safe) + "=" + urllib.quote(value[key], safe) for key in keys])
    else:
      return ",".join([urllib.quote(key, safe) + "," + urllib.quote(value[key], safe) for key in keys])
  else:
    if value:
      return varname + "=" + urllib.quote(value, safe)
    else:
      return varname 


def _tostring_query(varname, value, explode, operator, safe=""):
  joiner = operator
  if operator == "?":
    joiner = "&"
  if type(value) == type([]):
    if 0 == len(value):
      return ""
    if explode == "+":
      return joiner.join([varname + "=" + urllib.quote(x, safe) for x in value])
    elif explode == "*":
      return joiner.join([urllib.quote(x, safe) for x in value])
    else:
      return varname + "=" + ",".join([urllib.quote(x, safe) for x in value])
  elif type(value) == type({}):
    if 0 == len(value):
      return ""
    keys = value.keys()
    keys.sort()
    if explode == "+":
      return joiner.join([varname + "." + urllib.quote(key, safe) + "=" + urllib.quote(value[key], safe) for key in keys])
    elif explode == "*":
      return joiner.join([urllib.quote(key, safe) + "=" + urllib.quote(value[key], safe) for key in keys])
    else:
      return varname + "=" + ",".join([urllib.quote(key, safe) + "," + urllib.quote(value[key], safe) for key in keys])
  else:
    if value:
      return varname + "=" + urllib.quote(value, safe)
    else:
      return varname 

TOSTRING = {
    "" : _tostring,
    "+": _tostring,
    ";": _tostring_semi,
    "?": _tostring_query,
    "/": _tostring_path,
    ".": _tostring_path,
    }


def expand(template, vars):
  def _sub(match):
    expression = match.group(1)
    operator = ""
    if expression[0] in OPERATOR:
      operator = expression[0] 
      varlist = expression[1:]
    else:
      varlist = expression

    safe = ""
    explode = ""
    if operator == '+':
      safe = RESERVED
    varspecs = varlist.split(",")
    varnames = []
    defaults = {}
    for varspec in varspecs:
      default = None
      if "=" in varspec:
        varname, default = tuple(varspec.split("=", 1))
      else:
        varname = varspec
      if varname[-1] in EXPLODE:
        explode = varname[-1]
        varname = varname[:-1]
      if default:
        defaults[varname] = default
      varnames.append((varname, explode))

    retval = []
    joiner = operator
    prefix = operator
    if operator == "+":
      prefix = ""
      joiner = ","
    if operator == "?":
      joiner = "&"
    if operator == "":
      joiner = ","
    for varname, explode in varnames:
      if varname in vars:
        value = vars[varname]
        #if not value and (type(value) == type({}) or type(value) == type([])) and varname in defaults:
        if not value and value != "" and varname in defaults:
          value = defaults[varname]
      elif varname in defaults:
        value = defaults[varname]
      else:
        continue
      retval.append(TOSTRING[operator](varname, value, explode, operator, safe=safe))
    if "".join(retval):
      return prefix + joiner.join(retval)
    else:
      return ""

  return TEMPLATE.sub(_sub, template)



'''
Created on 13-09-2011

@author: piotrhol
'''

from httplib import HTTPConnection, CREATED, NO_CONTENT, OK, responses
import logging
import urllib2
import urlparse
import tempfile

log = logging.getLogger(__name__)

class RosrsApi:


    URI_ROS = "/ROs/"
    URI_RO_ID = URI_ROS + "%s/"
    URI_RESOURCE = URI_RO_ID + "%s"
    
    def __init__(self, rosrs_uri, accesstoken = None):
        parsed = urlparse.urlparse(rosrs_uri)
        self.rosrs_path = parsed.path
        self.rosrs_host = parsed.netloc
        self.rosrs_uri = rosrs_uri
        self.accesstoken = accesstoken
    
    
    def getRos(self):
        """
        Get a list of RO URIs. If called with an access token, returns 
        only ROs that belong to the user.
        
        Parameters: ROSRS URL, [access token]
        """
        conn = HTTPConnection(self.rosrs_host)
        url = self.rosrs_path + urllib2.quote(self.URI_ROS)
        body = None
        headers = self.__getAuthHeader()
        conn.request("GET", url, body, headers)
        res = conn.getresponse()
        if res.status != OK:
            raise Exception("%d %s: %s" % (res.status, res.reason, res.read()))
        roUris = res.read().splitlines()
        log.debug("RO list retrieved: %d items" % len(roUris))
        return roUris
    
    def postRo(self, roId):
        """
        Create a new Research Object in ROSRS.
        
        Parameters: ROSRS URL, username, password, RO id
        """
        conn = HTTPConnection(self.rosrs_host)
        url = self.rosrs_path + urllib2.quote(self.URI_ROS)
        body = roId
        headers = self.__getAuthHeader()
        headers["Content-Type"] = "text/plain"
        conn.request("POST", url, body, headers)
        res = conn.getresponse()
        if res.status != CREATED:
            raise Exception("%d %s: %s" % (res.status, res.reason, res.read()))
        log.debug("RO %s created: %s" % (roId, res.msg["location"]))
        return res.msg["location"]
    
    def deleteRo(self, roId):
        """
        Deletes a Research Object from ROSRS.
        
        Parameters: ROSRS URL, access token, RO id
        """
        roUrl = self.rosrs_path + urllib2.quote(self.URI_RO_ID % roId)
        return self.deleteRoByUrl(roUrl) 
    
    def deleteRoByUrl(self, roUrl):
        """
        Deletes a Research Object from ROSRS.
        
        Parameters: ROSRS URL, access token, RO URL
        """
        conn = HTTPConnection(self.rosrs_host)
        headers = self.__getAuthHeader()
        conn.request("DELETE", roUrl, None, headers)
        res = conn.getresponse()
        if res.status != NO_CONTENT:
            raise Exception("%d %s: %s" % (res.status, res.reason, res.read()))
        log.debug("RO %s deleted" % roUrl)
        return None 

    def getRoAsZip(self, roId):
        """
        Retrieves a Research Object version from ROSRS as a zip.
        
        Parameters: ROSRS URL, username, password, RO id, version id
        """
        url = self.rosrs_uri + urllib2.quote(self.URI_RO_ID % roId)
        return self.getRoAsZipByUrl(url)

    def getRoAsZipByUrl(self, roUrl):
        """
        Retrieves a Research Object version from ROSRS as a zip.
        
        Parameters: ROSRS URL, username, password, RO id, version id
        """
        req = urllib2.Request(roUrl)
        req.add_header("Accept", "application/zip")
        res = urllib2.urlopen(req)
        
        tmp = tempfile.TemporaryFile()
        while True:
            packet = res.read()
            if not packet:
                break
            tmp.write(packet)
        res.close()

        log.debug("Ro %s retrieved as zip" % roUrl)
        return tmp

    def putFile(self, roId, filePath, contentType, fileObject):
        """
        Creates or updates a file in ROSRS. This includes all metadata.
        
        Parameters: ROSRS URL, access token, RO id, file path (without trailing slash!),
        content type, file object
        """
        conn = HTTPConnection(self.rosrs_host)
        url = self.rosrs_path + urllib2.quote(self.URI_RESOURCE % (roId, filePath))
        body = fileObject
        headers = self.__getAuthHeader()
        headers["Content-Type"] = contentType or "text/plain"
        conn.request("PUT", url, body, headers)
        res = conn.getresponse()
        if res.status not in [OK,CREATED]:
            raise Exception("%d %s: %s" % (res.status, res.reason, res.read()))
        log.debug("File created/updated: %s" % filePath)
        return None
    
    def deleteFile(self, roId, filePath):
        """
        Deletes a file from ROSRS. This includes all metadata.
        
        Parameters: ROSRS URL, access token, RO id, file path
        """
        conn = HTTPConnection(self.rosrs_host)
        url = self.rosrs_path + urllib2.quote(self.URI_RESOURCE % (roId, filePath))
        headers = self.__getAuthHeader()
        conn.request("DELETE", url, None, headers)
        res = conn.getresponse()
        if res.status != NO_CONTENT:
            raise Exception("%d %s: %s" % (res.status, responses[res.status], res.reason))
        log.debug("File %s deleted" % filePath)
        return None
    
    def __getAuthHeader(self):
        if self.accesstoken == None:
            return dict()
        else:
            return {"Authorization": "Bearer %s" % self.accesstoken}
 
    

if __name__ == '__main__':
        pass



#!/usr/bin/python

"""
Module to test RO metadata handling class
"""

import os, os.path
import sys
import re
import shutil
import unittest
import logging
import datetime
import StringIO
try:
    # Running Python 2.5 with simplejson?
    import simplejson as json
except ImportError:
    import json

log = logging.getLogger(__name__)

if __name__ == "__main__":
    # Add main project directory and ro manager directories at start of python path
    sys.path.insert(0, "../..")
    sys.path.insert(0, "..")

import rdflib

from MiscLib import TestUtils

from rocommand import ro_settings
from rocommand import ro_metadata
from rocommand.ro_namespaces import RDF, RO, ORE, DCTERMS, ROTERMS
from rocommand.ro_annotation import annotationTypes

from TestConfig import ro_test_config
from StdoutContext import SwitchStdout

import TestROSupport

# Base directory for RO tests in this module
testbase = os.path.dirname(__file__)

# Local ro_config for testing
ro_config = {
    "annotationTypes": annotationTypes
    }

cwd        = os.getcwd()
robase     = ro_test_config.ROBASEDIR
robase_abs = os.path.abspath(ro_test_config.ROBASEDIR)

class TestROMetadata(TestROSupport.TestROSupport):
    """
    Test ro metadata handling
    """
    def setUp(self):
        super(TestROMetadata, self).setUp()
        return

    def tearDown(self):
        super(TestROMetadata, self).tearDown()
        return

    # Actual tests follow

    def testNull(self):
        assert True, 'Null test failed'

    def testCreateGraphRoUri(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test graph", "ro-testRoGraph")
        romd  = ro_metadata.ro_metadata(ro_config, rodir)
        self.assertEquals(romd.rouri, rdflib.URIRef("file://%s/RO_test_graph/"%(robase_abs)))
        self.deleteTestRo(rodir)
        return

    def testUpdateGraph(self):
        # No separate test - subsumed by other tests?
        return

    def testCreateReadRoAnnotationBody(self):
        """
        Test function to create & read a simple annotation body on an RO
        """
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        romd  = ro_metadata.ro_metadata(ro_config, rodir)
        roresource = "."
        attrdict = {
            "type":         "Research Object",
            # @@TODO: handle lists "keywords":     ["test", "research", "object"],
            "description":  "Test research object",
            "format":       "application/vnd.wf4ever.ro",
            "note":         "Research object created for annotation testing",
            "title":        "Test research object",
            "created":      "2011-12-07"
            }
        annotationfile = romd._createAnnotationBody(roresource, attrdict)
        # Ann-%04d%02d%02d-%04d-%s.rdf
        self.assertRegexpMatches(annotationfile,
            r"^\.ro/Ann-\d\d\d\d\d\d\d\d-\d+-RO_test_annotation\.rdf",
            msg="Unexpected filename form for annotation: "+annotationfile)
        annotationgraph = romd._readAnnotationBody(annotationfile)
        attrpropdict = {
            "type":         DCTERMS.type,
            # @@TODO "keywords":     DCTERMS.subject,
            "description":  DCTERMS.description,
            "format":       DCTERMS.format,
            "note":         ROTERMS.note,
            "title":        DCTERMS.title,
            "created":      DCTERMS.created
            }
        s = romd.getComponentUri(roresource)
        log.debug("annotation subject %s"%repr(s))
        for k in attrpropdict:
            p = attrpropdict[k]
            log.debug("annotation predicate %s"%repr(p))
            v = attrdict[k]
            a = annotationgraph.value(s, p, None)
            log.debug("annotation value %s"%repr(a))
            #self.assertEqual(len(a), 1, "Singleton result expected")
            self.assertEqual(a, v)
        self.deleteTestRo(rodir)
        return

    def testCreateReadFileAnnotationBody(self):
        """
        Test function to create & read a simple annotation body on a component file of an RO
        """
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        romd  = ro_metadata.ro_metadata(ro_config, rodir)
        roresource = "subdir1/subdir1-file.txt"
        attrdict = {
            "type":         "Test file",
            "description":  "File in test research object",
            "note":         "File in research object created for annotation testing",
            "title":        "Test file in RO",
            "created":      "2011-12-07"
            }
        annotationfile = romd._createAnnotationBody(roresource, attrdict)
        # Ann-%04d%02d%02d-%04d-%s.rdf
        self.assertRegexpMatches(annotationfile,
            r"^\.ro/Ann-\d\d\d\d\d\d\d\d-\d+-subdir1-file\.txt\.rdf",
            msg="Unexpected filename form for annotation: "+annotationfile)
        annotationgraph = romd._readAnnotationBody(annotationfile)
        attrpropdict = {
            "type":         DCTERMS.type,
            "description":  DCTERMS.description,
            "note":         ROTERMS.note,
            "title":        DCTERMS.title,
            "created":      DCTERMS.created
            }
        s = romd.getComponentUri(roresource)
        log.debug("annotation subject %s"%repr(s))
        for k in attrpropdict:
            p = attrpropdict[k]
            log.debug("annotation predicate %s"%repr(p))
            v = attrdict[k]
            a = annotationgraph.value(s, p, None)
            log.debug("annotation value %s"%repr(a))
            #self.assertEqual(len(a), 1, "Singleton result expected")
            self.assertEqual(a, v)
        self.deleteTestRo(rodir)
        return

    def testGetInitialRoAnnotations(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1",
            "Test init RO annotation", "ro-testRoAnnotate")
        romd  = ro_metadata.ro_metadata(ro_config, rodir)
        roresource = "."
        # Retrieve the anotations
        annotations = romd.getRoAnnotations()
        rouri = romd.getRoUri()
        expected_annotations = (
            [ (rouri, DCTERMS.description,  rdflib.Literal('Test init RO annotation'))
            , (rouri, DCTERMS.title,        rdflib.Literal('Test init RO annotation'))
            , (rouri, DCTERMS.created,      rdflib.Literal('unknown'))
            , (rouri, DCTERMS.creator,      rdflib.Literal('Test User'))
            , (rouri, DCTERMS.identifier,   rdflib.Literal('ro-testRoAnnotate'))
            , (rouri, RDF.type,             RO.ResearchObject)
            ])
        for i in range(6+1):      # Annotations + aggregations
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations and
                 next[1] != DCTERMS.created       and
                 next[1] != ORE.aggregates        ):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        self.deleteTestRo(rodir)
        return

    def testAddGetRoAnnotations(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1",
            "Test add RO annotation", "ro-testRoAnnotate")
        romd  = ro_metadata.ro_metadata(ro_config, rodir)
        roresource = "."
        # Add anotations for RO
        romd.addSimpleAnnotation(roresource, "type", "Research Object")
        romd.addSimpleAnnotation(roresource, "note", "Research object for annotation testing")
        romd.addSimpleAnnotation(roresource, "description", "Added description")
        # Retrieve the anotations
        annotations = romd.getRoAnnotations()
        rouri = romd.getRoUri()
        expected_annotations = (
            [ (rouri, DCTERMS.description,  rdflib.Literal('Test add RO annotation'))
            , (rouri, DCTERMS.title,        rdflib.Literal('Test add RO annotation'))
            , (rouri, DCTERMS.created,      rdflib.Literal('unknown'))
            , (rouri, DCTERMS.creator,      rdflib.Literal('Test User'))
            , (rouri, DCTERMS.identifier,   rdflib.Literal('ro-testRoAnnotate'))
            , (rouri, RDF.type,             RO.ResearchObject)
            , (rouri, DCTERMS.type,         rdflib.Literal('Research Object'))
            , (rouri, ROTERMS.note,         rdflib.Literal('Research object for annotation testing'))
            , (rouri, DCTERMS.description,  rdflib.Literal('Added description'))
            ])
        for i in range(9+4):      # Annotations + aggregations
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations and
                 next[1] != DCTERMS.created       and
                 next[1] != ORE.aggregates        ):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        self.deleteTestRo(rodir)
        return

    def testRemoveGetRoAnnotations(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1",
            "Test remove RO annotation", "ro-testRoAnnotate")
        romd  = ro_metadata.ro_metadata(ro_config, rodir)
        roresource = "."
        # Remove some anotations for RO
        romd.removeSimpleAnnotation(roresource, "type", "Research Object")
        romd.removeSimpleAnnotation(roresource, "title", "Test remove RO annotation")
        romd.removeSimpleAnnotation(roresource, "description",  None)
        romd.removeSimpleAnnotation(roresource, "note", "Research object for annotation testing")
        romd.removeSimpleAnnotation(roresource, "created", None)
        # Retrieve the anotations
        annotations = romd.getRoAnnotations()
        rouri = romd.getRoUri()
        expected_annotations = (
            [ (rouri, DCTERMS.creator,      rdflib.Literal('Test User'))
            , (rouri, DCTERMS.identifier,   rdflib.Literal('ro-testRoAnnotate'))
            , (rouri, RDF.type,             RO.ResearchObject)
            ])
        for i in range(4):
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations and
                 next[1] != DCTERMS.created       and
                 next[1] != ORE.aggregates        ):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        self.deleteTestRo(rodir)
        return

    def testReplaceGetRoAnnotations(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1",
            "Test replace RO annotation", "ro-testRoAnnotate")
        romd  = ro_metadata.ro_metadata(ro_config, rodir)
        roresource = "."
        # Replace anotations for RO
        romd.replaceSimpleAnnotation(roresource, "type",        "Research Object")
        romd.replaceSimpleAnnotation(roresource, "description", "Replacement description")
        romd.replaceSimpleAnnotation(roresource, "note", "Research object for annotation replacement testing")
        romd.replaceSimpleAnnotation(roresource, "title",       "Replacement title")
        romd.replaceSimpleAnnotation(roresource, "created",     "2011-12-07")
        # Retrieve the anotations
        annotations = romd.getRoAnnotations()
        rouri = romd.getRoUri()
        expected_annotations = (
            [ (rouri, DCTERMS.type,         rdflib.Literal('Research Object'))
            , (rouri, DCTERMS.title,        rdflib.Literal('Replacement title'))
            , (rouri, DCTERMS.description,  rdflib.Literal('Replacement description'))
            , (rouri, ROTERMS.note,         rdflib.Literal('Research object for annotation replacement testing'))
            , (rouri, DCTERMS.created,      rdflib.Literal('2011-12-07'))
            , (rouri, DCTERMS.creator,      rdflib.Literal('Test User'))
            , (rouri, DCTERMS.identifier,   rdflib.Literal('ro-testRoAnnotate'))
            , (rouri, RDF.type,             RO.ResearchObject)
            ])
        for i in range(8+1):      # Annotations + aggregations
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations and
                 next[1] != ORE.aggregates        ):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        self.deleteTestRo(rodir)
        return

    def testAddGetFileAnnotations(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1", "Test add file annotation", "ro-testRoAnnotate")
        romd  = ro_metadata.ro_metadata(ro_config, rodir)
        roresource = "subdir1/subdir1-file.txt"
        # Add anotations for file
        romd.addSimpleAnnotation(roresource, "type",        "Test file")
        romd.addSimpleAnnotation(roresource, "description", "File in test research object")
        romd.addSimpleAnnotation(roresource, "note", "Research object file created for annotation testing")
        romd.addSimpleAnnotation(roresource, "title",       "Test file in RO")
        romd.addSimpleAnnotation(roresource, "created",     "2011-12-07")
        romd.addSimpleAnnotation(roresource, "rdf:type",    ROTERMS.resource)
        # Retrieve the file anotations
        annotations = romd.getFileAnnotations(roresource)
        resourceuri = romd.getComponentUri(roresource)
        log.debug("resourceuri: %s"%(resourceuri))
        expected_annotations = (
            [ (resourceuri, DCTERMS.type,         rdflib.Literal('Test file'))
            , (resourceuri, DCTERMS.description,  rdflib.Literal('File in test research object'))
            , (resourceuri, ROTERMS.note,         rdflib.Literal('Research object file created for annotation testing'))
            , (resourceuri, DCTERMS.title,        rdflib.Literal('Test file in RO'))
            , (resourceuri, DCTERMS.created,      rdflib.Literal('2011-12-07'))
            , (resourceuri, RDF.type,             ROTERMS.resource)
            ])
        for i in range(6):
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        self.deleteTestRo(rodir)
        return

    def testRemoveGetFileAnnotations(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1",
            "Test remove file annotation", "ro-testRoAnnotate")
        romd  = ro_metadata.ro_metadata(ro_config, rodir)
        roresource = "subdir1/subdir1-file.txt"
        # Add anotations for file
        romd.addSimpleAnnotation(roresource, "type",        "Test file")
        romd.addSimpleAnnotation(roresource, "description", "File in test research object")
        romd.addSimpleAnnotation(roresource, "note", "Research object file created for annotation testing")
        romd.addSimpleAnnotation(roresource, "title",       "Test file in RO")
        romd.addSimpleAnnotation(roresource, "created",     "2011-12-07")
        romd.addSimpleAnnotation(roresource, "rdf:type",    ROTERMS.resource)
        # Remove annotations
        romd.removeSimpleAnnotation(roresource, "description", "File in test research object")
        romd.removeSimpleAnnotation(roresource, "note",        None)
        # Retrieve the file anotations
        annotations = romd.getFileAnnotations(roresource)
        resourceuri = romd.getComponentUri(roresource)
        log.debug("resourceuri: %s"%(resourceuri))
        expected_annotations = (
            [ (resourceuri, DCTERMS.type,         rdflib.Literal('Test file'))
            , (resourceuri, DCTERMS.title,        rdflib.Literal('Test file in RO'))
            , (resourceuri, DCTERMS.created,      rdflib.Literal('2011-12-07'))
            , (resourceuri, RDF.type,             ROTERMS.resource)
            ])
        for i in range(4):
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        self.deleteTestRo(rodir)
        return

    def testAddGetAllAnnotations(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1",
            "Test get all annotations", "ro-testRoAnnotate")
        romd  = ro_metadata.ro_metadata(ro_config, rodir)
        roresource = "subdir1/subdir1-file.txt"
        # Add anotations for file
        romd.addSimpleAnnotation(roresource, "type",        "Test file")
        romd.addSimpleAnnotation(roresource, "description", "File in test research object")
        romd.addSimpleAnnotation(roresource, "note", "Research object file created for annotation testing")
        romd.addSimpleAnnotation(roresource, "title",       "Test file in RO")
        romd.addSimpleAnnotation(roresource, "created",     "2011-12-07")
        romd.addSimpleAnnotation(roresource, "rdf:type",    ROTERMS.resource)
        # Retrieve the file anotations
        annotations = romd.getAllAnnotations()
        rouri       = romd.getRoUri()
        resourceuri = romd.getComponentUri(roresource)
        log.debug("resourceuri: %s"%(resourceuri))
        expected_annotations = (
            [ (rouri,       DCTERMS.description,  rdflib.Literal('Test get all annotations'))
            , (rouri,       DCTERMS.title,        rdflib.Literal('Test get all annotations'))
            , (rouri,       DCTERMS.created,      rdflib.Literal('unknown'))
            , (rouri,       DCTERMS.creator,      rdflib.Literal('Test User'))
            , (rouri,       DCTERMS.identifier,   rdflib.Literal('ro-testRoAnnotate'))
            , (rouri,       RDF.type,             RO.ResearchObject)
            , (resourceuri, DCTERMS.type,         rdflib.Literal('Test file'))
            , (resourceuri, DCTERMS.description,  rdflib.Literal('File in test research object'))
            , (resourceuri, ROTERMS.note,         rdflib.Literal('Research object file created for annotation testing'))
            , (resourceuri, DCTERMS.title,        rdflib.Literal('Test file in RO'))
            , (resourceuri, DCTERMS.created,      rdflib.Literal('2011-12-07'))
            , (resourceuri, RDF.type,             ROTERMS.resource)
            ])
        for i in range(12+1+6):      # Annotations + aggregations
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations and
                 next[1] != DCTERMS.created       and
                 next[1] != ORE.aggregates        ):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        self.deleteTestRo(rodir)
        return

    def testAddGetAnnotationValues(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1",
            "Test get annotation values", "ro-testRoAnnotate")
        romd  = ro_metadata.ro_metadata(ro_config, rodir)
        roresource = "subdir1/subdir1-file.txt"
        # Add anotations for file
        romd.addSimpleAnnotation(roresource, "type",         "Test file")
        romd.addSimpleAnnotation(roresource, "description",  "File in test research object")
        romd.addSimpleAnnotation(roresource, "rdf:type",     ROTERMS.resource)
        # Retrieve the anotations
        values = romd.getAnnotationValues(".", "title")
        self.assertEquals(values.next(), rdflib.Literal('Test get annotation values'))
        self.assertRaises(StopIteration, values.next)
        values = romd.getAnnotationValues(".", "rdf:type")
        self.assertEquals(values.next(), RO.ResearchObject)
        self.assertRaises(StopIteration, values.next)
        values = romd.getAnnotationValues(roresource, "type")
        self.assertEquals(values.next(), rdflib.Literal('Test file'))
        self.assertRaises(StopIteration, values.next)
        values = romd.getAnnotationValues(roresource, "description")
        self.assertEquals(values.next(), rdflib.Literal('File in test research object'))
        self.assertRaises(StopIteration, values.next)
        values = romd.getAnnotationValues(roresource, "rdf:type")
        self.assertEquals(values.next(), ROTERMS.resource)
        self.assertRaises(StopIteration, values.next)
        # Clean up
        self.deleteTestRo(rodir)
        return

    def testAddAggregatedResources(self):
        """
        Test function that adds aggregated resources to a research object manifest
        """
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test aggregation", "ro-testRoAggregation")
        romd  = ro_metadata.ro_metadata(ro_config, rodir)
        romd.addAggregatedResources(rodir, recurse=True)
        def URIRef(path):
            return romd.getComponentUri(path)
        s = romd.getRoUri()
        g = rdflib.Graph()
        g.add( (s, RDF.type,            RO.ResearchObject                  ) )
        g.add( (s, ORE.aggregates,      URIRef("README-ro-test-1")         ) )
        g.add( (s, ORE.aggregates,      URIRef("subdir1/subdir1-file.txt") ) )
        g.add( (s, ORE.aggregates,      URIRef("subdir2/subdir2-file.txt") ) )
        self.checkManifestGraph(rodir, g)
        self.deleteTestRo(rodir)
        return

    def testGetAggregatedResources(self):
        """
        Test function that enumerates aggregated resources to a research object manifest
        """
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test aggregation", "ro-testRoAggregation")
        romd  = ro_metadata.ro_metadata(ro_config, rodir)
        romd.addAggregatedResources(rodir, recurse=True)
        def URIRef(path):
            return romd.getComponentUri(path)
        resources = (
          [ URIRef("README-ro-test-1")
          , URIRef("minim.rdf")
          , URIRef("subdir1/subdir1-file.txt")
          , URIRef("subdir2/subdir2-file.txt")
          ])
        c = 0
        for r in romd.getAggregatedResources():
            if romd.getResourceType(r) != RO.AggregatedAnnotation:
                c += 1
                self.assertIn(r, resources)
        self.assertEqual(c, len(resources))
        self.deleteTestRo(rodir)
        return

    def testQueryAnnotations(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1",
            "Test query annotations", "ro-testRoAnnotate")
        romd  = ro_metadata.ro_metadata(ro_config, rodir)
        roresource = "subdir1/subdir1-file.txt"
        # Add anotations for file
        romd.addSimpleAnnotation(roresource, "type",        "Test file")
        romd.addSimpleAnnotation(roresource, "description", "File in test research object")
        romd.addSimpleAnnotation(roresource, "note", "Research object file created for annotation testing")
        romd.addSimpleAnnotation(roresource, "title",       "Test file in RO")
        romd.addSimpleAnnotation(roresource, "created",     "2011-12-07")
        romd.addSimpleAnnotation(roresource, "rdf:type",    ROTERMS.resource)
        # Query the file anotations
        queryprefixes = """
            PREFIX rdf:        <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
            PREFIX ro:         <http://purl.org/wf4ever/ro#>
            PREFIX ore:        <http://www.openarchives.org/ore/terms/>
            PREFIX ao:         <http://purl.org/ao/>
            PREFIX dcterms:    <http://purl.org/dc/terms/>
            PREFIX roterms:    <http://ro.example.org/ro/terms/>
            """
        query = (queryprefixes +
            """
            ASK
            {
                ?ro rdf:type ro:ResearchObject ;
                    dcterms:creator "Test User" .
                ?file rdf:type roterms:resource ;
                    dcterms:type "Test file" ;
                    dcterms:created "2011-12-07" .
            }
            """)
        resp = romd.queryAnnotations(query)
        self.assertTrue(resp, "Expected 'True' result for query: %s")
        query = (queryprefixes +
            """
            ASK
            {
                ?ro rdf:type ro:ResearchObject ;
                    dcterms:creator "Not user" .
            }
            """)
        resp = romd.queryAnnotations(query)
        self.assertFalse(resp, "Expected 'False' result for query: %s")
        query = (queryprefixes +
            """
            SELECT * WHERE
            {
                ?ro rdf:type ro:ResearchObject ;
                    dcterms:creator "Test User" .
                ?file rdf:type roterms:resource ;
                    dcterms:type "Test file" ;
                    dcterms:created ?date .
            }
            """)
        rouri       = romd.getRoUri()
        resourceuri = romd.getComponentUri(roresource)
        resp = romd.queryAnnotations(query)
        self.assertEqual(resp[0]['ro'],   rouri)
        self.assertEqual(resp[0]['file'], romd.getComponentUri(roresource))
        self.assertEqual(resp[0]['date'], rdflib.Literal("2011-12-07"))
        self.deleteTestRo(rodir)
        return

    def testQueryAnnotationsWithMissingGraph(self):
        """
        This test is included to ensure that queries still work as expected when
        the manifest contains a reference to a non-existent manifest graph
        """
        rodir = self.createTestRo(testbase, "data/ro-test-1",
            "Test query annotations", "ro-testRoAnnotate")
        romd  = ro_metadata.ro_metadata(ro_config, rodir)
        roresource = "subdir1/subdir1-file.txt"
        # Add anotations for file
        romd.addSimpleAnnotation(roresource, "type",        "Test file")
        romd.addSimpleAnnotation(roresource, "description", "File in test research object")
        romd.addSimpleAnnotation(roresource, "note", "Research object file created for annotation testing")
        romd.addSimpleAnnotation(roresource, "title",       "Test file in RO")
        romd.addSimpleAnnotation(roresource, "created",     "2011-12-07")
        romd.addSimpleAnnotation(roresource, "rdf:type",    ROTERMS.resource)
        # Apply non-exietent graph annotation
        annotation_graph_filename = os.path.join(os.path.abspath(rodir), "annotate-none.rdf")
        romd.addGraphAnnotation(roresource, annotation_graph_filename)
        # Query the file anotations
        queryprefixes = """
            PREFIX rdf:        <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
            PREFIX ro:         <http://purl.org/wf4ever/ro#>
            PREFIX ore:        <http://www.openarchives.org/ore/terms/>
            PREFIX ao:         <http://purl.org/ao/>
            PREFIX dcterms:    <http://purl.org/dc/terms/>
            PREFIX roterms:    <http://ro.example.org/ro/terms/>
            """
        query = (queryprefixes +
            """
            ASK
            {
                ?ro rdf:type ro:ResearchObject ;
                    dcterms:creator "Test User" .
                ?file rdf:type roterms:resource ;
                    dcterms:type "Test file" ;
                    dcterms:created "2011-12-07" .
            }
            """)
        resp = romd.queryAnnotations(query)
        self.assertTrue(resp, "Expected 'True' result for query: %s")
        self.deleteTestRo(rodir)
        return

    def testQueryAnnotationsRemote(self):
        romd  = ro_metadata.ro_metadata(
            ro_config,
            "http://andros.zoo.ox.ac.uk/workspace/wf4ever-ro-catalogue/v0.1/simple-requirements/"
            )
        # Query the file anotations
        queryprefixes = """
            PREFIX rdf:        <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
            PREFIX ro:         <http://purl.org/wf4ever/ro#>
            PREFIX ore:        <http://www.openarchives.org/ore/terms/>
            PREFIX ao:         <http://purl.org/ao/>
            PREFIX dcterms:    <http://purl.org/dc/terms/>
            PREFIX roterms:    <http://ro.example.org/ro/terms/>
            """
        query = (queryprefixes +
            """
            ASK
            {
                ?ro rdf:type ro:ResearchObject ;
                    dcterms:creator "Test user" ;
                    ore:aggregates ?file .
            }
            """)
        resp = romd.queryAnnotations(query)
        self.assertTrue(resp, "Expected 'True' result for query: %s")
        query = (queryprefixes +
            """
            ASK
            {
                ?ro rdf:type ro:ResearchObject ;
                    dcterms:creator "Not user" .
            }
            """)
        resp = romd.queryAnnotations(query)
        self.assertFalse(resp, "Expected 'False' result for query: %s")
        query = (queryprefixes +
            """
            SELECT * WHERE
            {
                ?ro rdf:type ro:ResearchObject ;
                    dcterms:creator "Test user" ;
                    ore:aggregates ?file .
            }
            """)
        rouri       = romd.getRoUri()
        resourceuri = romd.getComponentUri("README")
        resp = romd.queryAnnotations(query)
        self.assertEqual(resp[0]['ro'],   rouri)
        aggs = [ respn['file'] for respn in resp ]
        self.assertIn( resourceuri, aggs, repr(aggs))
        return

    # URI tests

    def testGetRoUri(self):
        def testUri(rodir, uristring):
            romd  = ro_metadata.ro_metadata(ro_config, rodir, dummysetupfortest=True)
            self.assertEquals(romd.getRoUri(), rdflib.URIRef(uristring))
            return
        testUri("/example/ro/dir",  "file:///example/ro/dir/" )
        testUri("/example/ro/dir/", "file:///example/ro/dir/" )
        testUri("ro/dir",           "file://%s/ro/dir/"%(cwd) )
        testUri("ro/dir/",          "file://%s/ro/dir/"%(cwd) )
        testUri(robase+"/ro/dir",   "file://%s/ro/dir/"%(robase_abs) )
        testUri(robase+"/ro/dir/",  "file://%s/ro/dir/"%(robase_abs) )
        return

    def testGetComponentUri(self):
        def testUri(rodir, path, uristring):
            romd  = ro_metadata.ro_metadata(ro_config, rodir, dummysetupfortest=True)
            self.assertEquals(romd.getComponentUri(path), rdflib.URIRef(uristring))
            return
        testUri("/example/ro/dir",  "a/b.txt", "file:///example/ro/dir/a/b.txt" )
        testUri("/example/ro/dir/", "a/b.txt", "file:///example/ro/dir/a/b.txt" )
        testUri("ro/dir",           "a/b.txt", "file://%s/ro/dir/a/b.txt"%(cwd) )
        testUri("ro/dir/",          "a/b.txt", "file://%s/ro/dir/a/b.txt"%(cwd) )
        testUri("/example/ro/dir",  "a/b/d/",  "file:///example/ro/dir/a/b/d/"  )
        testUri("/example/ro/dir/", "a/b/d/",  "file:///example/ro/dir/a/b/d/"  )
        testUri("ro/dir",           "a/b/d/",  "file://%s/ro/dir/a/b/d/"%(cwd)  )
        testUri("ro/dir/",          "a/b/d/",  "file://%s/ro/dir/a/b/d/"%(cwd)  )
        return

    def testGetComponentUriRel(self):
        def testUri(rodir, path, uristring):
            romd  = ro_metadata.ro_metadata(ro_config, rodir, dummysetupfortest=True)
            self.assertEquals(romd.getComponentUriRel(path), rdflib.URIRef(uristring))
            return

        testUri("/example/ro/dir",  "a/b.txt",  "a/b.txt" )
        testUri("/example/ro/dir/", "a/b.txt",  "a/b.txt" )
        testUri("ro/dir",           "a/b.txt",  "a/b.txt" )
        testUri("ro/dir/",          "a/b.txt",  "a/b.txt" )
        testUri("/example/ro/dir",  "a/b/d/",   "a/b/d/"  )
        testUri("/example/ro/dir/", "a/b/d/",   "a/b/d/"  )
        testUri("ro/dir",           "a/b/d/",   "a/b/d/"  )
        testUri("ro/dir/",          "a/b/d/",   "a/b/d/"  )
        testUri("/example/ro/dir",  "",         ""        )
        testUri("/example/ro/dir/", "",         ""        )
        testUri("ro/dir",           "",         ""        )
        testUri("ro/dir/",          "",         ""        )

        testUri("/example/ro/dir",  "/example/ro/dir/a/b.txt",  "a/b.txt" )
        testUri("/example/ro/dir/", "/example/ro/dir/a/b.txt",  "a/b.txt" )
        testUri("ro/dir",           "%s/ro/dir/a/b.txt"%(cwd),  "a/b.txt" )
        testUri("ro/dir/",          "%s/ro/dir/a/b.txt"%(cwd),  "a/b.txt" )
        testUri("/example/ro/dir",  "/example/ro/dir/a/b/d/",   "a/b/d/" )
        testUri("/example/ro/dir/", "/example/ro/dir/a/b/d/",   "a/b/d/" )
        testUri("ro/dir",           "%s/ro/dir/a/b/d/"%(cwd),   "a/b/d/" )
        testUri("ro/dir/",          "%s/ro/dir/a/b/d/"%(cwd),   "a/b/d/" )
        testUri("/example/ro/dir",  "/example/ro/dir/",         "" )
        testUri("/example/ro/dir/", "/example/ro/dir/",         "" )
        testUri("ro/dir",           "%s/ro/dir/"%(cwd),         "" )
        testUri("ro/dir/",          "%s/ro/dir/"%(cwd),         "" )

        testUri("/example/ro/dir",  "file:///example/ro/dir/a/b.txt",   "a/b.txt" )
        testUri("/example/ro/dir/", "file:///example/ro/dir/a/b.txt",   "a/b.txt" )
        testUri("/example/ro/dir",  "file:///example/ro/dir/a/b/d/",    "a/b/d/" )
        testUri("/example/ro/dir/", "file:///example/ro/dir/a/b/d/",    "a/b/d/" )
        testUri("/example/ro/dir",  "file:///example/ro/dir/",          "" )
        testUri("/example/ro/dir/", "file:///example/ro/dir/",          "" )
        return

    def testGetComponentUriRelUri(self):
        """
        Same as previous tests, but with path supplied as rdflib.URIRef value rather than string
        """
        def testUri(rodir, path, uristring):
            romd  = ro_metadata.ro_metadata(ro_config, rodir, dummysetupfortest=True)
            self.assertEquals(romd.getComponentUriRel(rdflib.URIRef(path)), rdflib.URIRef(uristring))
            return

        testUri("/example/ro/dir",  "a/b.txt",  "a/b.txt" )
        testUri("/example/ro/dir/", "a/b.txt",  "a/b.txt" )
        testUri("ro/dir",           "a/b.txt",  "a/b.txt" )
        testUri("ro/dir/",          "a/b.txt",  "a/b.txt" )
        testUri("/example/ro/dir",  "a/b/d/",   "a/b/d/"  )
        testUri("/example/ro/dir/", "a/b/d/",   "a/b/d/"  )
        testUri("ro/dir",           "a/b/d/",   "a/b/d/"  )
        testUri("ro/dir/",          "a/b/d/",   "a/b/d/"  )
        testUri("/example/ro/dir",  "",         ""        )
        testUri("/example/ro/dir/", "",         ""        )
        testUri("ro/dir",           "",         ""        )
        testUri("ro/dir/",          "",         ""        )

        testUri("/example/ro/dir",  "/example/ro/dir/a/b.txt",  "a/b.txt" )
        testUri("/example/ro/dir/", "/example/ro/dir/a/b.txt",  "a/b.txt" )
        testUri("ro/dir",           "%s/ro/dir/a/b.txt"%(cwd),  "a/b.txt" )
        testUri("ro/dir/",          "%s/ro/dir/a/b.txt"%(cwd),  "a/b.txt" )
        testUri("/example/ro/dir",  "/example/ro/dir/a/b/d/",   "a/b/d/" )
        testUri("/example/ro/dir/", "/example/ro/dir/a/b/d/",   "a/b/d/" )
        testUri("ro/dir",           "%s/ro/dir/a/b/d/"%(cwd),   "a/b/d/" )
        testUri("ro/dir/",          "%s/ro/dir/a/b/d/"%(cwd),   "a/b/d/" )
        testUri("/example/ro/dir",  "/example/ro/dir/",         "" )
        testUri("/example/ro/dir/", "/example/ro/dir/",         "" )
        testUri("ro/dir",           "%s/ro/dir/"%(cwd),         "" )
        testUri("ro/dir/",          "%s/ro/dir/"%(cwd),         "" )

        testUri("/example/ro/dir",  "file:///example/ro/dir/a/b.txt",   "a/b.txt" )
        testUri("/example/ro/dir/", "file:///example/ro/dir/a/b.txt",   "a/b.txt" )
        testUri("/example/ro/dir",  "file:///example/ro/dir/a/b/d/",    "a/b/d/" )
        testUri("/example/ro/dir/", "file:///example/ro/dir/a/b/d/",    "a/b/d/" )
        testUri("/example/ro/dir",  "file:///example/ro/dir/",          "" )
        testUri("/example/ro/dir/", "file:///example/ro/dir/",          "" )
        return

    # Sentinel/placeholder tests

    def testUnits(self):
        assert (True)

    def testComponents(self):
        assert (True)

    def testIntegration(self):
        assert (True)

    def testPending(self):
        assert (False), "Pending tests follow"

# Assemble test suite

def getTestSuite(select="unit"):
    """
    Get test suite

    select  is one of the following:
            "unit"      return suite of unit tests only
            "component" return suite of unit and component tests
            "all"       return suite of unit, component and integration tests
            "pending"   return suite of pending tests
            name        a single named test to be run
    """
    testdict = {
        "unit":
            [ "testUnits"
            , "testNull"
            , "testCreateGraphRoUri"
            , "testCreateReadRoAnnotationBody"
            , "testCreateReadFileAnnotationBody"
            , "testGetInitialRoAnnotations"
            , "testAddGetRoAnnotations"
            , "testRemoveGetRoAnnotations"
            , "testReplaceGetRoAnnotations"
            , "testAddGetFileAnnotations"
            , "testRemoveGetFileAnnotations"
            , "testAddGetAllAnnotations"
            , "testAddGetAnnotationValues"
            , "testQueryAnnotations"
            , "testQueryAnnotationsWithMissingGraph"
            , "testGetRoUri"
            , "testGetComponentUri"
            , "testGetComponentUriRel"
            , "testGetComponentUriRelUri"
            , "testAddAggregatedResources"
            , "testGetAggregatedResources"
            ],
        "component":
            [ "testComponents"
            ],
        "integration":
            [ "testIntegration"
            ],
        "pending":
            [ "testPending"
            , "testQueryAnnotationsRemote"
            ]
        }
    return TestUtils.getTestSuite(TestROMetadata, testdict, select=select)

if __name__ == "__main__":
    TestUtils.runTests("TestROMetadata.log", getTestSuite, sys.argv)

# End.



#!/usr/bin/env python

import sys, os, os.path

from MiscLib import TestUtils

def runTestSuite():
    """
    Transfer function for setup.py script ro-manager-test
    """
    base = os.path.dirname(__file__)
    os.chdir(base)
    #print "Run test suite assuming base path "+base
    sys.path.insert(0, os.path.normpath(base+"/..") )
    sys.path.insert(0, os.path.normpath(base+"/../..") )
    sys.path.insert(0, os.path.normpath(base+"/../../iaeval/test") )
    sys.path.insert(0, os.path.normpath(base+"/../../sync/test") )
    #print "Path: "+repr(sys.path)
    import TestAll
    TestUtils.runTests("TestAll", TestAll.getTestSuite, sys.argv)
    return 0



#!/usr/bin/python

"""
Module to test ROSRS synchronization RO manager commands

See: http://www.wf4ever-project.org/wiki/display/docs/RO+management+tool
"""

import sys
import os.path
import filecmp
import logging
import random
import string
try:
    # Running Python 2.5 with simplejson?
    import simplejson as json
except ImportError:
    import json
    
from os import remove, path

log = logging.getLogger(__name__)

if __name__ == "__main__":
    # Add main project directory and ro manager directories at start of python path
    sys.path.insert(0, "../..")
    sys.path.insert(0, "..")

from MiscLib import TestUtils
from sync.RosrsApi import RosrsApi
from sync.ResourceSync import ResourceSync

from rocommand import ro, ro_command
from TestConfig import ro_test_config
from StdoutContext import SwitchStdout

import TestROSupport

# Base directory for RO tests in this module
testbase = os.path.dirname(os.path.abspath(__file__))

class TestSyncCommands(TestROSupport.TestROSupport):
    """
    Test sync ro commands
    """
    
    files = ["subdir1/subdir1-file.txt"
             , "subdir2/subdir2-file.txt"
             , "README-ro-test-1"
             , ".ro/manifest.rdf"
             , "minim.rdf" ]

    fileToModify = "subdir1/subdir1-file.txt"
    fileToDelete = "README-ro-test-1"


    def setUp(self):
        super(TestSyncCommands, self).setUp()
        self.ident1 = ''.join(random.choice(string.ascii_uppercase + string.digits) for x in range(16))
        self.ident2 = ''.join(random.choice(string.ascii_uppercase + string.digits) for x in range(16))
        self.rodir = self.createTestRo(testbase, "data/ro-test-1", self.ident1, self.ident1)
        self.rodir2 = self.createTestRo(testbase, "data/ro-test-1", self.ident2, self.ident2)
        return

    def tearDown(self):
        super(TestSyncCommands, self).tearDown()
        self.deleteTestRo(self.rodir)
        self.deleteTestRo(self.rodir2)
        try:
            api = RosrsApi(ro_test_config.ROSRS_URI, ro_test_config.ROSRS_ACCESS_TOKEN)
            ros = api.getRos()
            for ro in ros:
                try:
                    api.deleteRoByUrl(ro)
                except:
                    pass
        except:
            pass
        return

    # Actual tests follow

    def testNull(self):
        assert True, 'Null test failed'

    def testPushAllForce(self):
        """
        Push all Research Objects to ROSRS, even unchanged.

        ro push [ -d <dir> ] [ -f ] [ -r <rosrs_uri> ] [ -t <access_token> ]
        """
        
        args = [
            "ro", "push",
            "-v",
            "-f"
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        assert status == 0
        self.assertEqual(self.outstr.getvalue().count("ro push"), 1)
        # must send each file exactly once
        for f in self.files:
            self.assertEqual(self.outstr.getvalue().count(self.rodir + "/" + f), 1)
        return

    def testPushAll(self):
        """
        Push all Research Objects to ROSRS.

        ro push [ -d <dir> ] [ -f ] [ -r <rosrs_uri> ] [ -t <access_token> ]
        """

        # first, send all        
        args = [
            "ro", "push",
            "-f"
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        assert status == 0
        
        # touch one file in 1st RO
        with open(path.join(self.rodir, self.fileToModify), 'a') as f:
            f.write("foobar")
            f.close()
        # delete one file in 2nd RO
        remove(path.join(self.rodir2, self.fileToDelete))

        # push again
        args = [
            "ro", "push",
            "-v"
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        assert status == 0
        self.assertEqual(self.outstr.getvalue().count("Updated:"), 1)
        self.assertEqual(self.outstr.getvalue().count("Deleted:"), 1)
        return

    def testPushOneRO(self):
        """
        Push a Research Object to ROSRS.

        ro push [ -d <dir> ] [ -f ] [ -r <rosrs_uri> ] [ -t <access_token> ]
        """
        # first, send all        
        args = [
            "ro", "push",
            "-f"
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        assert status == 0
        
        # touch one file in 1st RO
        with open(path.join(self.rodir, self.fileToModify), 'a') as f:
            f.write("foobar")
            f.close()
        # delete one file in 2nd RO
        remove(path.join(self.rodir2, self.fileToDelete))

        # push again, only 1st
        args = [
            "ro", "push",
            "-d", self.rodir,
            "-v"
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        assert status == 0
        self.assertEqual(self.outstr.getvalue().count("Updated:"), 1)
        self.assertEqual(self.outstr.getvalue().count("Deleted:"), 0)
        return

    def testCheckout(self):
        """
        Checkout a Research Object from ROSRS.

        ro checkout [ <RO-identifier> ] [ -d <dir> ] [ -r <rosrs_uri> ] [ -t <access_token> ]
        """
        # push an RO
        args = [
            "ro", "push",
            "-d", self.rodir,
            "-f"
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        assert status == 0
        
        # check it out as a copy
        rodir2 = os.path.join(ro_test_config.ROBASEDIR, "RO_test_checkout_copy")
        os.rename(self.rodir, rodir2)
        args = [
            "ro", "checkout", self.ident1,
            "-d", ro_test_config.ROBASEDIR,
            "-v"
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        assert status == 0
        self.assertEqual(self.outstr.getvalue().count("ro checkout"), 1)
        for f in self.files:
            self.assertEqual(self.outstr.getvalue().count(f), 1)
        self.assertEqual(self.outstr.getvalue().count("%d files checked out" % len(self.files)), 1)
        
        # compare they're identical, with the exception of registries.pickle
        cmpres = filecmp.dircmp(rodir2, self.rodir)
        self.assertEquals(cmpres.left_only, [ResourceSync.REGISTRIES_FILE])
        self.assertEquals(cmpres.right_only, [])
        self.assertListEqual(cmpres.diff_files, [], "Files should be the same (manifest is ignored)")

        # delete the checked out copy
        self.deleteTestRo(rodir2)
        return

    def testCheckoutAll(self):
        """
        Checkout a Research Object from ROSRS.

        ro checkout [ <RO-identifier> ] [ -d <dir>] [ -r <rosrs_uri> ] [ -t <access_token> ]
        """
        args = [
            "ro", "push",
            "-d", self.rodir,
            "-f"
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        assert status == 0
        
        rodir2 = os.path.join(ro_test_config.ROBASEDIR, self.ident2)
        args = [
            "ro", "checkout",
            "-d", ro_test_config.ROBASEDIR,
            "-v"
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        assert status == 0
        self.assertEqual(self.outstr.getvalue().count("ro checkout"), 1)
        for f in self.files:
            self.assertGreaterEqual(self.outstr.getvalue().count(f), 1, "counting %s" % f)
        self.assertGreaterEqual(self.outstr.getvalue().count("%d files checked out" % len(self.files)), 1)
        
        cmpres = filecmp.dircmp(self.rodir, rodir2)
        self.assertEquals(cmpres.left_only, [ResourceSync.REGISTRIES_FILE])
        self.assertEquals(cmpres.right_only, [])
        self.assertListEqual(cmpres.diff_files, [], "Files should be the same (manifest is ignored)")

        self.deleteTestRo(rodir2)
        return

    # Sentinel/placeholder tests

    def testUnits(self):
        assert (True)

    def testComponents(self):
        assert (True)

    def testIntegration(self):
        assert (True)

    def testPending(self):
        assert (False), "Pending tests follow"

# Assemble test suite

def getTestSuite(select="unit"):
    """
    Get test suite

    select  is one of the following:
            "unit"      return suite of unit tests only
            "component" return suite of unit and component tests
            "all"       return suite of unit, component and integration tests
            "pending"   return suite of pending tests
            name        a single named test to be run
    """
    testdict = {
        "unit":
            [ "testUnits"
            , "testNull"
            ],
        "component":
            [ "testComponents"
            , "testPushOneRO"
            #, "testPushAll"
            #, "testPushAllForce"
            , "testCheckout"
            , "testCheckoutAll"
            ],
        "integration":
            [ "testIntegration"
            ],
        "pending":
            [ "testPending"
            ]
        }
    return TestUtils.getTestSuite(TestSyncCommands, testdict, select=select)

if __name__ == "__main__":
    TestUtils.runTests("TestSyncCommands.log", getTestSuite, sys.argv)

# End.



# ro_namespaces.py

"""
Research Object manifest read, write, decode functions
"""

#import sys
#import os
#import os.path
#import re
#import urlparse
#import logging

import rdflib
import rdflib.namespace

import ro_settings

class Namespace(object):
    def __init__(self, baseUri):
        self.baseUri = baseUri
        return

def makeNamespace(baseUri, names):
    ns = Namespace(baseUri)
    for name in names:
        setattr(ns, name, rdflib.URIRef(baseUri+name))
    return ns

ao      = rdflib.URIRef("http://purl.org/ao/")
ore     = rdflib.URIRef("http://www.openarchives.org/ore/terms/")
foaf    = rdflib.URIRef("http://xmlns.com/foaf/0.1/")
ro      = rdflib.URIRef("http://purl.org/wf4ever/ro#")
wfprov  = rdflib.URIRef("http://purl.org/wf4ever/wfprov#")
wfdesc  = rdflib.URIRef("http://purl.org/wf4ever/wfdesc#")
dcterms = rdflib.URIRef("http://purl.org/dc/terms/")
roterms = rdflib.URIRef("http://ro.example.org/ro/terms/")

RDF     = makeNamespace(rdflib.namespace.RDF.uri,
            [ "Seq", "Bag", "Alt", "Statement", "Property", "XMLLiteral", "List", "PlainLiteral"
            , "subject", "predicate", "object", "type", "value", "first", "rest"
            , "nil"
            ])
RDFS    = makeNamespace(rdflib.namespace.RDFS.uri,
            [ "Resource", "Class", "subClassOf", "subPropertyOf", "comment", "label"
            , "domain", "range", "seeAlso", "isDefinedBy", "Literal", "Container"
            , "ContainerMembershipProperty", "member", "Datatype"
            ])
RO = makeNamespace(ro, 
            [ "ResearchObject", "AggregatedAnnotation"
            , "annotatesAggregatedResource"
            ])
ORE = makeNamespace(ore, 
            [ "Aggregation", "AggregatedResource"
            , "aggregates"
            ])
AO = makeNamespace(ao, 
            [ "Annotation"
            , "body"
            ])
DCTERMS = makeNamespace(dcterms, 
            [ "identifier", "description", "title", "creator", "created"
            , "subject", "format", "type"
            ])
ROTERMS = makeNamespace(roterms, 
            [ "note", "resource"
            ])

# End.



#!/usr/bin/python

"""
RO manager parameter settings
"""

import __init__
VERSION_NUM   = __init__.__version__

RO_VERSION    = "v"+VERSION_NUM
VERSION       = RO_VERSION+" (RODL)"
MANIFEST_DIR  = ".ro"
MANIFEST_FILE = "manifest.rdf"

# End.



#!/usr/bin/env python

import sys, unittest

if __name__ == "__main__":
    # Add main project directory and ro manager directories at start of python path
    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")
    sys.path.insert(0, "../../iaeval/test")
    sys.path.insert(0, "../../sync/test")

import TestConfig
import TestBasicCommands
import TestAnnotationUtils
import TestManifest
import TestROMetadata
import TestAnnotations
import TestLinks
import TestMinimAccess
import TestEvalChecklist
import TestRosrsApi
import TestResourceSync
import TestSyncCommands

# Code to run unit tests from all library test modules
def getTestSuite(select="unit"):
    suite = unittest.TestSuite()
    suite.addTest(TestBasicCommands.getTestSuite(select=select))
    suite.addTest(TestAnnotationUtils.getTestSuite(select=select))
    suite.addTest(TestManifest.getTestSuite(select=select))
    suite.addTest(TestROMetadata.getTestSuite(select=select))
    suite.addTest(TestAnnotations.getTestSuite(select=select))
    suite.addTest(TestLinks.getTestSuite(select=select))
    suite.addTest(TestMinimAccess.getTestSuite(select=select))
    suite.addTest(TestEvalChecklist.getTestSuite(select=select))
    if select != "unit":
        suite.addTest(TestRosrsApi.getTestSuite(select=select))
        suite.addTest(TestResourceSync.getTestSuite(select=select))
        suite.addTest(TestSyncCommands.getTestSuite(select=select))
    return suite

from MiscLib import TestUtils

def runTestSuite():
    """
    Transfer function for setup.py script ro-manager-test
    """
    base = os.path.dirname(__file__)
    #print "Run test suite assuming base path "+base
    sys.path.insert(0, os.path.normpath(base+"/..") )
    sys.path.insert(0, os.path.normpath(base+"/../..") )
    sys.path.insert(0, os.path.normpath(base+"/../../iaeval/test") )
    sys.path.insert(0, os.path.normpath(base+"/../../sync/test") )
    #print "Path: "+repr(sys.path)
    TestUtils.runTests("TestAll", getTestSuite, sys.argv)
    return 0

if __name__ == "__main__":
    print "By default, runs quick tests only."
    print "Use \"python TestAll.py all\" to run all tests"
    TestUtils.runTests("TestAll", getTestSuite, sys.argv)

# End.



'''
Created on 14-09-2011

@author: piotrhol
'''

import sys
if __name__ == "__main__":
    sys.path.append("../..")

from MiscLib import TestUtils
import os.path
import random
import string
from rocommand.test import TestROSupport
from sync.RosrsApi import RosrsApi
from rocommand.test.TestConfig import ro_test_config
from zipfile import ZipFile

# Base directory for RO tests in this module
testbase = os.path.dirname(os.path.abspath(__file__))

class TestRosrsApi(TestROSupport.TestROSupport):
    
    """
    Test ro annotation commands
    """
    def setUp(self):
        super(TestRosrsApi, self).setUp()
        self.setupConfig()
        self.ident1 = ''.join(random.choice(string.ascii_uppercase + string.digits) for x in range(16))
        self.ident2 = ''.join(random.choice(string.ascii_uppercase + string.digits) for x in range(16))
        self.rodir = self.createTestRo(testbase, "../../rocommand/test/data/ro-test-1", self.ident1, self.ident1)
        return

    def tearDown(self):
        super(TestRosrsApi, self).tearDown()
        self.deleteTestRo(self.rodir)
        try:
            api = RosrsApi(ro_test_config.ROSRS_URI, ro_test_config.ROSRS_ACCESS_TOKEN)
            ros = api.getRos()
            for ro in ros:
                try:
                    api.deleteRoByUrl(ro)
                except:
                    pass
        except:
            pass
        return

    # Setup local config for local tests

    def setupConfig(self):
        return self.setupTestBaseConfig(testbase)
    
    def testNull(self):
        assert True, 'Null test failed'

    def testROCreation(self):
        api = RosrsApi(ro_test_config.ROSRS_URI, ro_test_config.ROSRS_ACCESS_TOKEN)
        api.postRo(self.ident1)
        api.putFile(self.ident1, "folderX/fileY.txt", "text/plain", open(os.path.join(self.rodir, "subdir1/subdir1-file.txt")))
        api.deleteFile(self.ident1, "folderX/fileY.txt")
        api.deleteRo(self.ident1)
        
    def testGetRoAsZip(self):
        api = RosrsApi(ro_test_config.ROSRS_URI, ro_test_config.ROSRS_ACCESS_TOKEN)
        api.postRo(self.ident1)
        api.putFile(self.ident1, "folderX/fileY.txt", "text/plain", open(os.path.join(self.rodir, "subdir1/subdir1-file.txt")))
        verzip = api.getRoAsZip(self.ident1)
        zipfile = ZipFile(verzip)
        self.assertEquals(len(zipfile.read("folderX/fileY.txt")), os.path.getsize(os.path.join(self.rodir, "subdir1/subdir1-file.txt")), "FileY size must be the same")
        self.assertTrue(len(zipfile.read(".ro/manifest.rdf")) > 0, "Size of manifest.rdf must be greater than 0")
        api.deleteRo(self.ident1)
        return
    
    def testGetRos(self):
        api = RosrsApi(ro_test_config.ROSRS_URI, ro_test_config.ROSRS_ACCESS_TOKEN)
        api.postRo(self.ident1)
        api.postRo(self.ident2)
        ros = api.getRos()
        self.assertSetEqual(set(ros), { ro_test_config.ROSRS_URI + "/ROs/" + self.ident1 + "/", ro_test_config.ROSRS_URI + "/ROs/" + self.ident2 + "/" }, "Returned ROs are not correct")
        api.deleteRo(self.ident1)
        api.deleteRo(self.ident2)
        return
        
def getTestSuite(select="unit"):
    """
    Get test suite

    select  is one of the following:
            "unit"      return suite of unit tests only
            "component" return suite of unit and component tests
            "all"       return suite of unit, component and integration tests
            "pending"   return suite of pending tests
            name        a single named test to be run
    """
    testdict = {
        "unit":
            [ "testUnits"
            , "testNull"
            ],
        "component":
            [ "testComponents"
            , "testROCreation"
            , "testGetRoAsZip"
            , "testGetRos"
            ],
        "integration":
            [ "testIntegration"
            ],
        "pending":
            [ "testPending"
            ]
        }
    return TestUtils.getTestSuite(TestRosrsApi, testdict, select=select)

if __name__ == "__main__":
    TestUtils.runTests("TestRosrsApi.log", getTestSuite, sys.argv)



#!/usr/bin/python

"""
Module to test RO manager manifest and aggregation commands

See: http://www.wf4ever-project.org/wiki/display/docs/RO+management+tool
"""

import os, os.path
import sys
import re
import shutil
import unittest
import logging
import datetime
import StringIO
try:
    # Running Python 2.5 with simplejson?
    import simplejson as json
except ImportError:
    import json

log = logging.getLogger(__name__)

if __name__ == "__main__":
    # Add main project directory and ro manager directories at start of python path
    sys.path.insert(0, "../..")
    sys.path.insert(0, "..")

import rdflib

from MiscLib import TestUtils

from rocommand import ro
from rocommand import ro_utils
from rocommand import ro_manifest
from rocommand import ro_annotation
from rocommand.ro_namespaces import RDF, DCTERMS, RO, AO, ORE

from TestConfig import ro_test_config
from StdoutContext import SwitchStdout

import TestROSupport

# Base directory for RO tests in this module
testbase = os.path.dirname(__file__)

# Local ro_config for testing
#ro_config = {
#    "annotationTypes": ro_annotation.annotationTypes
#    }

class TestManifest(TestROSupport.TestROSupport):
    """
    Test ro annotation commands
    """
    def setUp(self):
        super(TestManifest, self).setUp()
        return

    def tearDown(self):
        super(TestManifest, self).tearDown()
        return

    # Actual tests follow

    def testNull(self):
        assert True, 'Null test failed'

    def testManifestContent(self):
        """
        Test content of newly created manifest
        """
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        ro_graph = ro_manifest.readManifestGraph(rodir)
        self.checkManifestGraph(rodir, ro_graph)
        s = ro_manifest.getComponentUri(rodir, "")
        g = rdflib.Graph()
        g.add( (s, RDF.type,            RO.ResearchObject                            ) )
        g.add( (s, DCTERMS.creator,     rdflib.Literal(ro_test_config.ROBOXUSERNAME) ) )
        g.add( (s, DCTERMS.description, rdflib.Literal("RO test annotation")         ) )
        g.add( (s, DCTERMS.title,       rdflib.Literal("RO test annotation")         ) )
        g.add( (s, DCTERMS.identifier,  rdflib.Literal("ro-testRoAnnotate")          ) )
        self.checkManifestGraph(rodir, g)
        self.deleteTestRo(rodir)
        return

    def testAddAggregatedResources(self):
        """
        Test function that adds aggregated resources to a research object manifest
        """
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        ro_manifest.addAggregatedResources(rodir, rodir, recurse=True)
        def URIRef(path):
            return ro_manifest.getComponentUri(rodir, path)
        s = ro_manifest.getComponentUri(rodir, "")
        g = rdflib.Graph()
        g.add( (s, RDF.type,            RO.ResearchObject                  ) )
        g.add( (s, ORE.aggregates,      URIRef("README-ro-test-1")         ) )
        g.add( (s, ORE.aggregates,      URIRef("subdir1/subdir1-file.txt") ) )
        g.add( (s, ORE.aggregates,      URIRef("subdir2/subdir2-file.txt") ) )
        self.checkManifestGraph(rodir, g)
        self.deleteTestRo(rodir)
        return

    def testAddAggregatedResourcesCommand(self):
        """
        Test function that adds aggregated resources to a research object manifest
        """
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        args = [
            "ro", "add", "-v", "-a",
            "-d", rodir,
            rodir
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        outtxt = self.outstr.getvalue()
        assert status == 0, outtxt
        log.debug("outtxt %s"%outtxt)
        def URIRef(path):
            return ro_manifest.getComponentUri(rodir, path)
        s = ro_manifest.getComponentUri(rodir, "")
        g = rdflib.Graph()
        g.add( (s, RDF.type,            RO.ResearchObject                  ) )
        g.add( (s, ORE.aggregates,      URIRef("README-ro-test-1")         ) )
        g.add( (s, ORE.aggregates,      URIRef("subdir1/subdir1-file.txt") ) )
        g.add( (s, ORE.aggregates,      URIRef("subdir2/subdir2-file.txt") ) )
        self.checkManifestGraph(rodir, g)
        self.deleteTestRo(rodir)
        return

    # Sentinel/placeholder tests

    def testUnits(self):
        assert (True)

    def testComponents(self):
        assert (True)

    def testIntegration(self):
        assert (True)

    def testPending(self):
        assert (False), "Pending tests follow"

# Assemble test suite

def getTestSuite(select="unit"):
    """
    Get test suite

    select  is one of the following:
            "unit"      return suite of unit tests only
            "component" return suite of unit and component tests
            "all"       return suite of unit, component and integration tests
            "pending"   return suite of pending tests
            name        a single named test to be run
    """
    testdict = {
        "unit":
            [ "testUnits"
            , "testNull"
            , "testManifestContent"
            , "testAddAggregatedResources"
            , "testAddAggregatedResourcesCommand"
            ],
        "component":
            [ "testComponents"
            ],
        "integration":
            [ "testIntegration"
            ],
        "pending":
            [ "testPending"
            ]
        }
    return TestUtils.getTestSuite(TestManifest, testdict, select=select)

if __name__ == "__main__":
    TestUtils.runTests("TestManifest.log", getTestSuite, sys.argv)

# End.



#!/usr/bin/python

"""
Module to test RO manager annotation commands

See: http://www.wf4ever-project.org/wiki/display/docs/RO+management+tool
"""

import os, os.path
import sys
import re
import shutil
import unittest
import logging
import datetime
import StringIO
try:
    # Running Python 2.5 with simplejson?
    import simplejson as json
except ImportError:
    import json

log = logging.getLogger(__name__)

if __name__ == "__main__":
    # Add main project directory and ro manager directories at start of python path
    sys.path.insert(0, "../..")
    sys.path.insert(0, "..")

import rdflib

from MiscLib import TestUtils

from rocommand import ro
from rocommand import ro_utils
from rocommand import ro_manifest
from rocommand import ro_annotation
from rocommand.ro_namespaces import RDF, DCTERMS, RO, AO, ORE

from TestConfig import ro_test_config
from StdoutContext import SwitchStdout

import TestROSupport

# Base directory for RO tests in this module
testbase = os.path.dirname(os.path.abspath(__file__))

# Local ro_config for testing
ro_config = {
    "annotationTypes": ro_annotation.annotationTypes
    }

def LIT(l): return rdflib.Literal(l)
def REF(u): return rdflib.URIRef(u)

class TestLinks(TestROSupport.TestROSupport):
    """
    Test ro annotation commands
    """
    def setUp(self):
        super(TestLinks, self).setUp()
        return

    def tearDown(self):
        super(TestLinks, self).tearDown()
        return

    # Actual tests follow

    def testNull(self):
        assert True, 'Null test failed'

    def testLink(self):
        """
        Annotate file in created RO

        ro annotate file attribute-name [ attribute-value ]
        """
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        args = [
            "ro", "annotate", rodir+"/"+"subdir1/subdir1-file.txt", "title", "subdir1-file title",
            "-v",
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        outtxt = self.outstr.getvalue()
        assert status == 0, outtxt
        log.debug("outtxt %s"%outtxt)
        #self.assertRegexpMatches(outtxt, "annotation.*dc:title")
        # Read manifest and check for annotation
        values = ro_annotation._getAnnotationValues(ro_config, rodir, "subdir1/subdir1-file.txt", "title")
        self.assertEquals(values.next(), rdflib.Literal("subdir1-file title"))
        self.assertRaises(StopIteration, values.next)
        # Clean up
        self.deleteTestRo(rodir)
        return

    def linkTest(self, anntype, annvalue, anntypeuri, annexpect):
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        args = [
            "ro", "link", rodir+"/"+"subdir1/subdir1-file.txt", anntype, annvalue,
            "-v",
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        outtxt = self.outstr.getvalue()
        assert status == 0, outtxt
        self.assertEqual(outtxt.count("ro annotate"), 1)
        # Read manifest and check for annotation
        annotations = ro_annotation._getFileAnnotations(rodir, "subdir1/subdir1-file.txt")
        resourceuri = ro_manifest.getComponentUri(rodir, "subdir1/subdir1-file.txt")
        expected_annotations = (
            [ (resourceuri, anntypeuri, annexpect)
            ])
        for i in range(1):
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        self.deleteTestRo(rodir)
        return

    # Other annotation types to add (cf. http://wf4ever.github.com/labs/ro-annotator/mockups/1/index.html)

    def testLinkType(self):
        self.linkTest("type", "file type", DCTERMS.type, LIT("file type"))
        return

    def testLinkCreated(self):
        created = "2011-09-14T12:00:00"
        self.linkTest("created", created, DCTERMS.created, LIT(created))
        return

    def testLinkRdfType(self):
        typeref = "http://example.com/type"
        self.linkTest("rdf:type", typeref, RDF.type, REF(typeref))
        return

    def testLinkUnknownUri(self):
        anntypeuri = "http://example.org/unknowntype"
        annvalue   = "http://example.com/link"
        self.linkTest(anntypeuri, annvalue, REF(anntypeuri), REF(annvalue))
        return

    # Test annotate with graph
    def testLinkWithGraph(self):
        rodir  = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        rofile = rodir+"/"+"subdir1/subdir1-file.txt"
        define_annotations = (
            [ {"atypename": "type",        "avalue":"atype",    "atypeuri":DCTERMS.type,        "aexpect":"atype" }
            , {"atypename": "keywords",    "avalue":"asubj",    "atypeuri":DCTERMS.subject,     "aexpect":"asubj" }
            , {"atypename": "description", "avalue":"adesc",    "atypeuri":DCTERMS.description, "aexpect":"adesc" }
            , {"atypename": "format",      "avalue":"aformat",  "atypeuri":DCTERMS.format,      "aexpect":"aformat" }
            , {"atypename": "title",       "avalue":"atitle",   "atypeuri":DCTERMS.title,       "aexpect":"atitle" }
            , {"atypename": "created",     "avalue":"acreated", "atypeuri":DCTERMS.created,     "aexpect":"acreated" }
            ])
        # Create annotation graph file and apply annotations
        annotation_graph = rdflib.Graph()
        resourceuri = ro_manifest.getComponentUri(rodir, "subdir1/subdir1-file.txt")
        for ann in define_annotations:
            annotation_graph.add( (resourceuri, ann["atypeuri"], rdflib.Literal(ann["aexpect"])) )
        annotation_graph_filename = os.path.join(os.path.abspath(rodir), "annotate-subdir1-file.txt.rdf")
        annotation_graph.serialize(annotation_graph_filename,
            format='xml', base=ro_manifest.getRoUri(rodir), xml_base="")
        args = ["ro", "annotate", rofile, "-g", annotation_graph_filename ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        outtxt = self.outstr.getvalue()
        assert status == 0, outtxt
        # Read manifest and check for annotation
        annotations = ro_annotation._getFileAnnotations(rodir, "subdir1/subdir1-file.txt")
        expected_annotations = (
            [ (resourceuri, a["atypeuri"], rdflib.Literal(a["aexpect"]))
                for a in define_annotations
            ])
        for i in range(6):
            next = annotations.next()
            if ( next not in expected_annotations):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        # Clean up
        self.deleteTestRo(rodir)
        return

    # Test annotate with non-existent graph (make nsure it doesn't all fall over)
    def testLinkWithNotExistentGraph(self):
        rodir  = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        rofile = rodir+"/"+"subdir1/subdir1-file.txt"
        # Apply non-exietent graph annotation
        annotation_graph_filename = os.path.join(os.path.abspath(rodir), "annotate-none.rdf")
        rouri = ro_manifest.getRoUri(rodir)
        args = ["ro", "annotate", rodir+"/", "-g", annotation_graph_filename ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(ro_test_config.CONFIGDIR, ro_test_config.ROBASEDIR, args)
        outtxt = self.outstr.getvalue()
        assert status == 0, outtxt
        # Read manifest and check for annotation
        annotations = ro_annotation.getAllAnnotations(rodir)
        expected_annotations = (
            [ ( rouri, DCTERMS.identifier,  rdflib.Literal('ro-testRoAnnotate')  )
            , ( rouri, DCTERMS.description, rdflib.Literal('RO test annotation') )
            , ( rouri, DCTERMS.title,       rdflib.Literal('RO test annotation') )
            , ( rouri, DCTERMS.creator,     rdflib.Literal('Test User') )
            , ( rouri, RDF.type,            RO.ResearchObject )
            ])
        for i in range(8):
            next = annotations.next()
            if (next not in expected_annotations
                and not isinstance(next[2], rdflib.BNode)
                and not next[1] == DCTERMS.created):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        # Clean up
        self.deleteTestRo(rodir)
        return

    # Sentinel/placeholder tests

    def testUnits(self):
        assert (True)

    def testComponents(self):
        assert (True)

    def testIntegration(self):
        assert (True)

    def testPending(self):
        assert (False), "Pending tests follow"

# Assemble test suite

def getTestSuite(select="unit"):
    """
    Get test suite

    select  is one of the following:
            "unit"      return suite of unit tests only
            "component" return suite of unit and component tests
            "all"       return suite of unit, component and integration tests
            "pending"   return suite of pending tests
            name        a single named test to be run
    """
    testdict = {
        "unit":
            [ "testUnits"
            , "testNull"
            , "testLink"
            , "testLinkType"
            , "testLinkCreated"
            , "testLinkRdfType"
            , "testLinkUnknownUri"
            , "testLinkWithGraph"
            , "testLinkWithNotExistentGraph"
            ],
        "component":
            [ "testComponents"
            ],
        "integration":
            [ "testIntegration"
            ],
        "pending":
            [ "testPending"
            ]
        }
    return TestUtils.getTestSuite(TestLinks, testdict, select=select)

if __name__ == "__main__":
    TestUtils.runTests("TestLinks.log", getTestSuite, sys.argv)

# End.
